import importlib
import logging
import pytest

from web3.auto import (
    infura,
)
from web3.providers import (
    HTTPProvider,
    IPCProvider,
    WebsocketProvider,
)
from web3.providers.auto import (
    load_provider_from_environment,
)


@pytest.mark.parametrize(
    'uri, expected_type, expected_attrs',
    (
        ('', type(None), {}),
        ('http://1.2.3.4:5678', HTTPProvider, {'endpoint_uri': 'http://1.2.3.4:5678'}),
        ('https://node.ontheweb.com', HTTPProvider, {'endpoint_uri': 'https://node.ontheweb.com'}),
        ('file:///root/path/to/file.ipc', IPCProvider, {'ipc_path': '/root/path/to/file.ipc'}),
        ('ws://1.2.3.4:5679', WebsocketProvider, {'endpoint_uri': 'ws://1.2.3.4:5679'})
    ),
)
def test_load_provider_from_env(monkeypatch, uri, expected_type, expected_attrs):
    monkeypatch.setenv('WEB3_PROVIDER_URI', uri)
    provider = load_provider_from_environment()
    assert isinstance(provider, expected_type)
    for attr, val in expected_attrs.items():
        assert getattr(provider, attr) == val


def test_web3_auto_infura_empty_key(monkeypatch, caplog):
    monkeypatch.setenv('INFURA_API_KEY', '')

    importlib.reload(infura)
    assert len(caplog.record_tuples) == 1
    logger, level, msg = caplog.record_tuples[0]
    assert 'INFURA_API_KEY' in msg
    assert level == logging.WARNING

    w3 = infura.w3
    assert isinstance(w3.providers[0], HTTPProvider)
    assert getattr(w3.providers[0], 'endpoint_uri') == infura.INFURA_MAINNET_BASE_URL


def test_web3_auto_infura_missing_key(monkeypatch, caplog):
    monkeypatch.delenv('INFURA_API_KEY', raising=False)

    importlib.reload(infura)
    assert len(caplog.record_tuples) == 1
    logger, level, msg = caplog.record_tuples[0]
    assert 'INFURA_API_KEY' in msg
    assert level == logging.WARNING

    w3 = infura.w3
    assert isinstance(w3.providers[0], HTTPProvider)
    assert getattr(w3.providers[0], 'endpoint_uri') == infura.INFURA_MAINNET_BASE_URL


def test_web3_auto_infura(monkeypatch, caplog):
    API_KEY = 'aoeuhtns'
    monkeypatch.setenv('INFURA_API_KEY', API_KEY)
    expected_url = '%s/%s' % (infura.INFURA_MAINNET_BASE_URL, API_KEY)

    importlib.reload(infura)
    assert len(caplog.record_tuples) == 0

    w3 = infura.w3
    assert isinstance(w3.providers[0], HTTPProvider)
    assert getattr(w3.providers[0], 'endpoint_uri') == expected_url



import json
import pytest

from eth_utils import (
    decode_hex,
)

from web3.contract import (
    Contract,
)
from web3.exceptions import (
    FallbackNotFound,
)


def test_class_construction_sets_class_vars(web3,
                                            MATH_ABI,
                                            MATH_CODE,
                                            MATH_RUNTIME):
    MathContract = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_CODE,
        bytecode_runtime=MATH_RUNTIME,
    )

    assert MathContract.web3 == web3
    assert MathContract.bytecode == decode_hex(MATH_CODE)
    assert MathContract.bytecode_runtime == decode_hex(MATH_RUNTIME)


def test_error_to_instantiate_base_class():
    with pytest.raises(AttributeError):
        Contract()


def test_abi_as_json_string(web3, MATH_ABI, some_address):
    abi_str = json.dumps(MATH_ABI)

    MathContract = web3.eth.contract(abi=abi_str)
    assert MathContract.abi == MATH_ABI

    math = MathContract(some_address)
    assert math.abi == MATH_ABI


def test_error_to_call_non_existent_fallback(web3,
                                             MATH_ABI,
                                             MATH_CODE,
                                             MATH_RUNTIME):
    math_contract = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_CODE,
        bytecode_runtime=MATH_RUNTIME,
    )
    with pytest.raises(FallbackNotFound):
        math_contract.fallback.estimateGas()



import pytest

from web3.utils.blocks import (
    select_method_for_block_identifier,
)
from web3.utils.toolz import (
    partial,
)

selector_fn = partial(
    select_method_for_block_identifier,
    if_hash='test_hash',
    if_number='test_number',
    if_predefined='test_predefined',
)


@pytest.mark.parametrize(
    'input,expected',
    (
        ('latest', 'test_predefined'),
        ('pending', 'test_predefined'),
        ('earliest', 'test_predefined'),
        (-1, ValueError),
        (0, 'test_number'),
        (1, 'test_number'),
        (4000000, 'test_number'),
        ('0x0', 'test_number'),
        ('0x00', 'test_number'),
        ('0x1', 'test_number'),
        ('0x01', 'test_number'),
        (hex(4000000), 'test_number'),
        ('0x' + ''.zfill(64), 'test_hash'),
    ),
)
def test_select_method_for_block_identifier(input, expected):
    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            selector_fn(input)
    else:
        actual = selector_fn(input)
        assert actual == expected



from web3.manager import (
    RequestManager,
)
from web3.providers import (
    BaseProvider,
)


def stateful_middleware(make_request, web3):
    state = []

    def middleware(method, params):
        state.append((method, params))
        return {'result': state}

    middleware.state = state
    return middleware


def test_middleware_holds_state_across_requests():
    provider = BaseProvider()

    manager = RequestManager(None, provider, middlewares=[stateful_middleware])
    state_a = manager.request_blocking('test_statefulness', [])
    assert len(state_a) == 1

    state_b = manager.request_blocking('test_statefulness', [])

    assert id(state_a) == id(state_b)
    assert len(state_b) == 2



from functools import (
    singledispatch,
)
import operator

from eth_account import (
    Account,
)
from eth_account.local import (
    LocalAccount,
)
from eth_keys.datatypes import (
    PrivateKey,
)
from eth_utils import (
    to_dict,
)

from web3.utils.formatters import (
    apply_formatter_if,
)
from web3.utils.toolz import (
    compose,
)
from web3.utils.transactions import (
    fill_nonce,
    fill_transaction_defaults,
)

to_hexstr_from_eth_key = operator.methodcaller('to_hex')


def is_eth_key(value):
    return isinstance(value, PrivateKey)


key_normalizer = compose(
    apply_formatter_if(is_eth_key, to_hexstr_from_eth_key),
)


@to_dict
def gen_normalized_accounts(val):
    if isinstance(val, (list, tuple, set,)):
        for i in val:
            account = to_account(i)
            yield account.address, account
    else:
        account = to_account(val)
        yield account.address, account
        return


@singledispatch
def to_account(val):
    raise TypeError(
        "key must be one of the types: "
        "eth_keys.datatype.PrivateKey, eth_account.local.LocalAccount, "
        "or raw private key as a hex string or byte string. "
        "Was of type {0}".format(type(val)))


@to_account.register(LocalAccount)
def _(val):
    return val


def private_key_to_account(val):
    normalized_key = key_normalizer(val)
    return Account.privateKeyToAccount(normalized_key)


to_account.register(PrivateKey, private_key_to_account)
to_account.register(str, private_key_to_account)
to_account.register(bytes, private_key_to_account)


def construct_sign_and_send_raw_middleware(private_key_or_account):
    """Capture transactions sign and send as raw transactions


    Keyword arguments:
    private_key_or_account -- A single private key or a tuple,
    list or set of private keys. Keys can be any of the following formats:
      - An eth_account.LocalAccount object
      - An eth_keys.PrivateKey object
      - A raw private key as a hex string or byte string
    """

    accounts = gen_normalized_accounts(private_key_or_account)

    def sign_and_send_raw_middleware(make_request, w3):

        fill_tx = compose(
            fill_transaction_defaults(w3),
            fill_nonce(w3))

        def middleware(method, params):
            if method != "eth_sendTransaction":
                return make_request(method, params)
            else:
                transaction = fill_tx(params[0])

            if 'from' not in transaction:
                return make_request(method, params)
            elif transaction.get('from') not in accounts:
                return make_request(method, params)

            account = accounts[transaction['from']]
            raw_tx = account.signTransaction(transaction).rawTransaction

            return make_request(
                "eth_sendRawTransaction",
                [raw_tx])

        return middleware

    return sign_and_send_raw_middleware



import pytest

from web3.utils.filters import (
    construct_data_filter_regex,
)


def hex_and_pad(*i):
    return '0x' + ''.join(
        hex(v).rstrip('L')[2:].zfill(64)
        for v in i
    )


SINGLE_VALUE = [
    [hex_and_pad(1)],
]
TWO_VALUES = [
    [hex_and_pad(12345), hex_and_pad(54321)],
]
TWO_VALUES_WITH_WILDCARD = [
    [hex_and_pad(12345), None],
]
THREE_VALUES_WITH_WILDCARD = [
    [hex_and_pad(12345), None, hex_and_pad(54321)],
]
MULTI_VALUE = [
    [hex_and_pad(1)],
    [hex_and_pad(2)],
]
MULTI_NESTED_VALUE = [
    [hex_and_pad(1, 2)],
    [hex_and_pad(2, 1)],
]
MULTI_NESTED_VALUE_WITH_WILDCARD = [
    [hex_and_pad(1), None],
    [hex_and_pad(2, 1)],
]


@pytest.mark.parametrize(
    "filter_set,data_value,should_match",
    (
        (SINGLE_VALUE, hex_and_pad(1), True),
        (SINGLE_VALUE, hex_and_pad(0), False),
        (SINGLE_VALUE, hex_and_pad(2), False),
        (TWO_VALUES, hex_and_pad(12345, 54321), True),
        (TWO_VALUES_WITH_WILDCARD, hex_and_pad(12345, 54321), True),
        (TWO_VALUES_WITH_WILDCARD, hex_and_pad(12345, 1), True),
        (TWO_VALUES_WITH_WILDCARD, hex_and_pad(1, 1), False),
        (THREE_VALUES_WITH_WILDCARD, hex_and_pad(12345, 0, 54321), True),
        (THREE_VALUES_WITH_WILDCARD, hex_and_pad(12345, 1, 54321), True),
        (THREE_VALUES_WITH_WILDCARD, hex_and_pad(12345, 2**256 - 1, 54321), True),
        (THREE_VALUES_WITH_WILDCARD, hex_and_pad(12344, 0, 54321), False),
        (MULTI_VALUE, hex_and_pad(1), True),
        (MULTI_VALUE, hex_and_pad(2), True),
        (MULTI_VALUE, hex_and_pad(3), False),
        (MULTI_VALUE, hex_and_pad(1, 2), True),
        (MULTI_VALUE, hex_and_pad(2, 1), False),
        (MULTI_NESTED_VALUE_WITH_WILDCARD, hex_and_pad(1, 1), True),
        (MULTI_NESTED_VALUE_WITH_WILDCARD, hex_and_pad(1, 12345), True),
        (MULTI_NESTED_VALUE_WITH_WILDCARD, hex_and_pad(2, 1), True),
        (MULTI_NESTED_VALUE_WITH_WILDCARD, hex_and_pad(2, 12345), False),
    )
)
def test_construct_data_filter_regex(filter_set, data_value, should_match):
    data_filter_regex = construct_data_filter_regex(filter_set)
    is_match = bool(data_filter_regex.match(data_value))
    assert is_match is should_match, data_filter_regex



import signal
import socket
import subprocess
import time

from eth_utils import (
    to_text,
)
import requests


def wait_for_socket(ipc_path, timeout=60):
    start = time.time()
    while time.time() < start + timeout:
        try:
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            sock.connect(ipc_path)
            sock.settimeout(timeout)
        except (FileNotFoundError, socket.error):
            time.sleep(0.01)
        else:
            break


def wait_for_http(endpoint_uri, timeout=60):
    start = time.time()
    while time.time() < start + timeout:
        try:
            requests.get(endpoint_uri)
        except requests.ConnectionError:
            time.sleep(0.01)
        else:
            break


def get_process(command_list, terminates=False):

    proc = subprocess.Popen(
        command_list,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        bufsize=1,
    )
    if terminates:
        wait_for_popen(proc, 30)
    try:
        yield proc
    finally:
        kill_proc_gracefully(proc)
        output, errors = proc.communicate()
        print(
            "Parity Process Exited:\n"
            "stdout:{0}\n\n"
            "stderr:{1}\n\n".format(
                to_text(output),
                to_text(errors),
            )
        )


def wait_for_popen(proc, timeout):
    start = time.time()
    while time.time() < start + timeout:
        if proc.poll() is None:
            time.sleep(0.01)
        else:
            break


def kill_proc_gracefully(proc):
    if proc.poll() is None:
        proc.send_signal(signal.SIGINT)
        wait_for_popen(proc, 13)

    if proc.poll() is None:
        proc.terminate()
        wait_for_popen(proc, 5)

    if proc.poll() is None:
        proc.kill()
        wait_for_popen(proc, 2)


def get_open_port():
    sock = socket.socket()
    sock.bind(('127.0.0.1', 0))
    port = sock.getsockname()[1]
    sock.close()
    return str(port)



import os
import pathlib
import pytest
import socket
import tempfile
from threading import (
    Thread,
)
import time
import uuid

from web3.auto.gethdev import (
    w3,
)
from web3.middleware import (
    construct_fixture_middleware,
)
from web3.providers.ipc import (
    IPCProvider,
)


@pytest.fixture
def jsonrpc_ipc_pipe_path():
    with tempfile.TemporaryDirectory() as temp_dir:
        ipc_path = os.path.join(temp_dir, '{0}.ipc'.format(uuid.uuid4()))
        try:
            yield ipc_path
        finally:
            if os.path.exists(ipc_path):
                os.remove(ipc_path)


def test_ipc_no_path():
    """
    IPCProvider.isConnected() returns False when no path is supplied
    """
    ipc = IPCProvider(None)
    assert ipc.isConnected() is False


@pytest.fixture
def simple_ipc_server(jsonrpc_ipc_pipe_path):
    serv = socket.socket(socket.AF_UNIX)
    serv.bind(jsonrpc_ipc_pipe_path)
    serv.listen(1)
    try:
        yield serv
    finally:
        serv.close()


@pytest.fixture
def serve_empty_result(simple_ipc_server):
    def reply():
        connection, client_address = simple_ipc_server.accept()
        try:
            connection.recv(1024)
            connection.sendall(b'{"id":1, "result": {}')
            time.sleep(0.1)
            connection.sendall(b'}')
        finally:
            # Clean up the connection
            connection.close()
            simple_ipc_server.close()

    thd = Thread(target=reply, daemon=True)
    thd.start()

    try:
        yield
    finally:
        thd.join()


def test_sync_waits_for_full_result(jsonrpc_ipc_pipe_path, serve_empty_result):
    provider = IPCProvider(pathlib.Path(jsonrpc_ipc_pipe_path), timeout=3)
    result = provider.make_request("method", [])
    assert result == {'id': 1, 'result': {}}
    provider._socket.sock.close()


def test_web3_auto_gethdev():
    assert isinstance(w3.providers[0], IPCProvider)
    return_block_with_long_extra_data = construct_fixture_middleware({
        'eth_getBlockByNumber': {'extraData': '0x' + 'ff' * 33},
    })
    w3.middleware_stack.inject(return_block_with_long_extra_data, layer=0)
    block = w3.eth.getBlock('latest')
    assert 'extraData' not in block
    assert block.proofOfAuthorityData == b'\xff' * 33



import pytest

from web3 import Web3
from web3.middleware import (
    construct_error_generator_middleware,
    construct_fixture_middleware,
    construct_result_generator_middleware,
)
from web3.providers.base import (
    BaseProvider,
)


class DummyProvider(BaseProvider):
    def make_request(self, method, params):
        raise NotImplementedError("Cannot make request for {0}:{1}".format(
            method,
            params,
        ))


@pytest.fixture
def w3():
    return Web3(providers=[DummyProvider()], middlewares=[])


@pytest.mark.parametrize(
    'method,expected',
    (
        ('test_endpoint', 'value-a'),
        ('not_implemented', NotImplementedError),
    )
)
def test_fixture_middleware(w3, method, expected):
    w3.middleware_stack.add(construct_fixture_middleware({'test_endpoint': 'value-a'}))

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            w3.manager.request_blocking(method, [])
    else:
        actual = w3.manager.request_blocking(method, [])
        assert actual == expected


@pytest.mark.parametrize(
    'method,expected',
    (
        ('test_endpoint', 'value-a'),
        ('not_implemented', NotImplementedError),
    )
)
def test_result_middleware(w3, method, expected):
    def _callback(method, params):
        return params[0]

    w3.middleware_stack.add(construct_result_generator_middleware({
        'test_endpoint': _callback,
    }))

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            w3.manager.request_blocking(method, [expected])
    else:
        actual = w3.manager.request_blocking(method, [expected])
        assert actual == expected


@pytest.mark.parametrize(
    'method,expected',
    (
        ('test_endpoint', 'value-a'),
        ('not_implemented', NotImplementedError),
    )
)
def test_error_middleware(w3, method, expected):
    def _callback(method, params):
        return params[0]

    w3.middleware_stack.add(construct_error_generator_middleware({
        'test_endpoint': _callback,
    }))

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            w3.manager.request_blocking(method, [expected])
    else:
        with pytest.raises(ValueError) as err:
            w3.manager.request_blocking(method, [expected])
        assert expected in str(err)




import pytest
from unittest.mock import (
    patch,
)

from eth_utils import (
    is_same_address,
    to_bytes,
)

from ens.constants import (
    EMPTY_ADDR_HEX,
)
from ens.main import (
    UnauthorizedError,
)
from web3 import Web3


'''
API at: https://github.com/carver/ens.py/issues/2
'''


@pytest.mark.parametrize(
    'name, full_name, namehash_hex',
    [
        (
            'tester.eth',
            'tester.eth',
            '0x2a7ac1c833d35677c2ff34a908951de142cc1653de6080ad4e38f4c9cc00aafe',
        ),
        (
            'tester',
            'tester.eth',
            '0x2a7ac1c833d35677c2ff34a908951de142cc1653de6080ad4e38f4c9cc00aafe',
        ),
        (
            'TESTER',
            'TESTER.eth',
            '0x2a7ac1c833d35677c2ff34a908951de142cc1653de6080ad4e38f4c9cc00aafe',
        ),
        # handles alternative dot separators
        (
            'tester．eth',
            'tester．eth',
            '0x2a7ac1c833d35677c2ff34a908951de142cc1653de6080ad4e38f4c9cc00aafe',
        ),
        (
            'tester。eth',
            'tester。eth',
            '0x2a7ac1c833d35677c2ff34a908951de142cc1653de6080ad4e38f4c9cc00aafe',
        ),
        (
            'tester｡eth',
            'tester｡eth',
            '0x2a7ac1c833d35677c2ff34a908951de142cc1653de6080ad4e38f4c9cc00aafe',
        ),
        # confirm that set-owner works
        (
            'lots.of.subdomains.tester.eth',
            'lots.of.subdomains.tester.eth',
            '0x0d62a759aa1f1c9680de8603a12a5eb175cd1bfa79426229868eba99f4dce692',
        ),
        (
            'lots.of.subdomains.tester',
            'lots.of.subdomains.tester.eth',
            '0x0d62a759aa1f1c9680de8603a12a5eb175cd1bfa79426229868eba99f4dce692',
        ),
    ],
)
def test_set_address(ens, name, full_name, namehash_hex, TEST_ADDRESS):
    assert ens.address(name) is None
    owner = ens.owner('tester')

    ens.setup_address(name, TEST_ADDRESS)
    assert is_same_address(ens.address(name), TEST_ADDRESS)

    # check that .eth is only appended if guess_tld is True
    namehash = Web3.toBytes(hexstr=namehash_hex)
    normal_name = ens.nameprep(full_name)
    if ens.nameprep(name) == normal_name:
        assert is_same_address(ens.address(name, guess_tld=False), TEST_ADDRESS)
    else:
        assert ens.address(name, guess_tld=False) is None

    # check that the correct namehash is set:
    assert is_same_address(ens.resolver(normal_name).addr(namehash), TEST_ADDRESS)

    # check that the correct owner is set:
    assert ens.owner(name) == owner

    ens.setup_address(name, None)
    assert ens.address(name) is None


@pytest.mark.parametrize(
    'name, equivalent',
    [
        ('TESTER', 'tester.eth'),
        ('unicÖde.tester.eth', 'unicöde.tester.eth'),
    ],
)
def test_set_address_equivalence(ens, name, equivalent, TEST_ADDRESS):
    assert ens.address(name) is None

    ens.setup_address(name, TEST_ADDRESS)
    assert is_same_address(ens.address(name), TEST_ADDRESS)
    assert is_same_address(ens.address(equivalent), TEST_ADDRESS)

    ens.setup_address(name, None)
    assert ens.address(name) is None


@pytest.mark.parametrize(
    'set_address',
    [
        # since the test uses getTransactionCount,
        # using a same address converted to bytes and hex will error with same count,
        # use two different addresses of each type (hex, bytes)
        "0x000000000000000000000000000000000000dEaD",
        to_bytes(hexstr="0x5B2063246F2191f18F2675ceDB8b28102e957458"),
        EMPTY_ADDR_HEX,
        None,
        '',
    ],
)
def test_set_address_noop(ens, set_address):
    eth = ens.web3.eth
    owner = ens.owner('tester.eth')
    ens.setup_address('noop.tester.eth', set_address)
    starting_transactions = eth.getTransactionCount(owner)

    # do not issue transaction if address is already set
    ens.setup_address('noop.tester.eth', set_address)
    assert eth.getTransactionCount(owner) == starting_transactions


def test_set_address_unauthorized(ens, TEST_ADDRESS):
    with pytest.raises(UnauthorizedError):
        ens.setup_address('eth', TEST_ADDRESS)


def test_setup_address_default_address_to_owner(ens):
    assert ens.address('default.tester.eth') is None
    owner = ens.owner('tester.eth')

    ens.setup_address('default.tester.eth')
    assert ens.address('default.tester.eth') == owner


def test_first_owner_upchain_identify(ens):
    # _first_owner should auto-select the name owner to send the transaction from
    addr = '0x5B2063246F2191f18F2675ceDB8b28102e957458'

    def getowner(name):
        if name == "cdefghi.eth":
            return addr
        else:
            return None
    with patch.object(ens, 'owner', side_effect=getowner):
        assert ens._first_owner('abcdefg.bcdefgh.cdefghi.eth') == \
            (addr, ['abcdefg', 'bcdefgh'], 'cdefghi.eth')


def test_set_resolver_leave_default(ens, TEST_ADDRESS):
    owner = ens.owner('tester')
    ens.setup_address('leave-default-resolver.tester.eth', TEST_ADDRESS)
    eth = ens.web3.eth
    num_transactions = eth.getTransactionCount(owner)

    ens.setup_address('leave-default-resolver.tester', '0x5B2063246F2191f18F2675ceDB8b28102e957458')

    # should skip setting the owner and setting the default resolver, and only
    #   set the name in the default resolver to point to the new address
    assert eth.getTransactionCount(owner) == num_transactions + 1



import pytest

from web3.utils.abi import (
    is_probably_enum,
)


@pytest.mark.parametrize(
    'abi_type,should_match',
    (
        ('SomeEnum.SomeValue', True),
        ('Some_Enum.Some_Value', True),
        ('SomeEnum.someValue', True),
        ('SomeEnum.some_value', True),
        ('__SomeEnum__.some_value', True),
        ('__SomeEnum__.__some_value__', True),
        ('SomeEnum.__some_value__', True),
        ('uint256', False),
    ),
)
def test_is_probably_enum(abi_type, should_match):
    is_match = is_probably_enum(abi_type)
    assert is_match is should_match



import random

from web3.utils.threads import (
    Timeout,
)


def test_txpool_inspect(web3_empty):
    web3 = web3_empty

    web3.miner.stop()

    with Timeout(60) as timeout:
        while web3.miner.hashrate or web3.eth.mining:
            timeout.sleep(random.random())

    txn_1_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 12345,
    })
    txn_1 = web3.eth.getTransaction(txn_1_hash)
    txn_2_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 54321,
    })
    txn_2 = web3.eth.getTransaction(txn_2_hash)

    inspect_content = web3.txpool.inspect

    assert web3.eth.coinbase in inspect_content['pending']

    pending_txns = inspect_content['pending'][web3.eth.coinbase]

    assert txn_1['nonce'] in pending_txns
    assert txn_2['nonce'] in pending_txns

    txn_1_summary = pending_txns[txn_1['nonce']][0]
    txn_2_summary = pending_txns[txn_2['nonce']][0]

    assert '0xd3CdA913deB6f67967B99D67aCDFa1712C293601' in txn_1_summary
    assert '12345 wei' in txn_1_summary

    assert '0xd3CdA913deB6f67967B99D67aCDFa1712C293601' in txn_2_summary
    assert '54321 wei' in txn_2_summary



# -*- coding: utf-8 -*-

import pytest

from eth_utils import (
    to_bytes,
)

from web3.utils.empty import (
    empty,
)

# Ignore warning in pyethereum 1.6 - will go away with the upgrade
pytestmark = pytest.mark.filterwarnings("ignore:implicit cast from 'char *'")


@pytest.fixture()
def math_contract(web3, MathContract, address_conversion_func):
    deploy_txn = MathContract.constructor().transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    address = address_conversion_func(deploy_receipt['contractAddress'])
    _math_contract = MathContract(address=address)
    assert _math_contract.address == address
    return _math_contract


@pytest.fixture()
def string_contract(web3, StringContract, address_conversion_func):
    deploy_txn = StringContract.constructor("Caqalai").transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    address = address_conversion_func(deploy_receipt['contractAddress'])
    _string_contract = StringContract(address=address)
    assert _string_contract.address == address
    return _string_contract


@pytest.fixture()
def fallback_function_contract(web3, FallballFunctionContract, address_conversion_func):
    deploy_txn = FallballFunctionContract.constructor().transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    address = address_conversion_func(deploy_receipt['contractAddress'])
    _fallback_contract = FallballFunctionContract(address=address)
    assert _fallback_contract.address == address
    return _fallback_contract


@pytest.fixture()
def arrays_contract(web3, ArraysContract, address_conversion_func):
    # bytes_32 = [keccak('0'), keccak('1')]
    bytes32_array = [
        b'\x04HR\xb2\xa6p\xad\xe5@~x\xfb(c\xc5\x1d\xe9\xfc\xb9eB\xa0q\x86\xfe:\xed\xa6\xbb\x8a\x11m',  # noqa: E501
        b'\xc8\x9e\xfd\xaaT\xc0\xf2\x0cz\xdfa(\x82\xdf\tP\xf5\xa9Qc~\x03\x07\xcd\xcbLg/)\x8b\x8b\xc6',  # noqa: E501
    ]
    byte_arr = [b'\xff', b'\xff', b'\xff', b'\xff']
    deploy_txn = ArraysContract.constructor(bytes32_array, byte_arr).transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    address = address_conversion_func(deploy_receipt['contractAddress'])
    _arrays_contract = ArraysContract(address=address)
    return _arrays_contract


def test_transacting_with_contract_no_arguments(web3, math_contract, transact, call):
    initial_value = call(contract=math_contract,
                         contract_function='counter')

    txn_hash = transact(contract=math_contract,
                        contract_function='increment')
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt is not None

    final_value = call(contract=math_contract,
                       contract_function='counter')

    assert final_value - initial_value == 1


@pytest.mark.parametrize(
    'transact_args,transact_kwargs',
    (
        ((5,), {}),
        (tuple(), {'amt': 5}),
    ),
)
def test_transacting_with_contract_with_arguments(web3,
                                                  math_contract,
                                                  transact,
                                                  call,
                                                  transact_args,
                                                  transact_kwargs):
    initial_value = call(contract=math_contract,
                         contract_function='counter')

    txn_hash = transact(contract=math_contract,
                        contract_function='increment',
                        func_args=transact_args,
                        func_kwargs=transact_kwargs)

    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt is not None

    final_value = call(contract=math_contract,
                       contract_function='counter')

    assert final_value - initial_value == 5


def test_deploy_when_default_account_is_set(web3,
                                            wait_for_transaction,
                                            STRING_CONTRACT):
    web3.eth.defaultAccount = web3.eth.accounts[1]
    assert web3.eth.defaultAccount is not empty

    StringContract = web3.eth.contract(**STRING_CONTRACT)

    deploy_txn = StringContract.constructor("Caqalai").transact()
    web3.eth.waitForTransactionReceipt(deploy_txn)
    txn_after = web3.eth.getTransaction(deploy_txn)
    assert txn_after['from'] == web3.eth.defaultAccount


def test_transact_when_default_account_is_set(web3,
                                              wait_for_transaction,
                                              math_contract,
                                              transact):
    web3.eth.defaultAccount = web3.eth.accounts[1]
    assert web3.eth.defaultAccount is not empty

    txn_hash = transact(contract=math_contract,
                        contract_function='increment')
    wait_for_transaction(web3, txn_hash)
    txn_after = web3.eth.getTransaction(txn_hash)
    assert txn_after['from'] == web3.eth.defaultAccount


def test_transacting_with_contract_with_string_argument(web3, string_contract, transact, call):
    # eth_abi will pass as raw bytes, no encoding
    # unless we encode ourselves
    txn_hash = transact(contract=string_contract,
                        contract_function='setValue',
                        func_args=["ÄLÄMÖLÖ".encode('utf8')])
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt is not None

    final_value = call(contract=string_contract,
                       contract_function='getValue')

    assert final_value == "ÄLÄMÖLÖ"


def test_transacting_with_contract_with_bytes32_array_argument(web3,
                                                               arrays_contract,
                                                               transact,
                                                               call):
    # new_bytes32_array = [keccak('1'), keccak('2'), keccak('3')]
    new_bytes32_array = [
        b'\xc8\x9e\xfd\xaaT\xc0\xf2\x0cz\xdfa(\x82\xdf\tP\xf5\xa9Qc~\x03\x07\xcd\xcbLg/)\x8b\x8b\xc6',  # noqa: E501
        b'\xad|[\xef\x02x\x16\xa8\x00\xda\x176DO\xb5\x8a\x80~\xf4\xc9`;xHg?~:h\xeb\x14\xa5',
        b"*\x80\xe1\xef\x1dxB\xf2\x7f.k\xe0\x97+\xb7\x08\xb9\xa15\xc3\x88`\xdb\xe7<'\xc3Hl4\xf4\xde",  # noqa: E501
    ]
    txn_hash = transact(contract=arrays_contract,
                        contract_function="setBytes32Value",
                        func_args=[new_bytes32_array])
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt is not None

    final_value = call(contract=arrays_contract,
                       contract_function="getBytes32Value")
    assert final_value == new_bytes32_array


def test_transacting_with_contract_with_byte_array_argument(web3, arrays_contract, transact, call):
    new_byte_array = [b'\x03', b'\x03', b'\x03', b'\x03', b'\x03', b'\x03']
    txn_hash = transact(contract=arrays_contract,
                        contract_function='setByteValue',
                        func_args=[new_byte_array])
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt is not None

    final_value = call(contract=arrays_contract,
                       contract_function='getByteValue')
    assert final_value == new_byte_array


def test_transacting_with_contract_respects_explicit_gas(web3,
                                                         STRING_CONTRACT,
                                                         skip_if_testrpc,
                                                         wait_for_block,
                                                         call,
                                                         transact):
    skip_if_testrpc(web3)

    wait_for_block(web3)

    StringContract = web3.eth.contract(**STRING_CONTRACT)

    deploy_txn = StringContract.constructor("Caqalai").transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn, 30)
    assert deploy_receipt is not None
    string_contract = StringContract(address=deploy_receipt['contractAddress'])

    # eth_abi will pass as raw bytes, no encoding
    # unless we encode ourselves
    txn_hash = transact(contract=string_contract,
                        contract_function='setValue',
                        func_args=[to_bytes(text="ÄLÄMÖLÖ")],
                        tx_kwargs={'gas': 200000})
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash, 30)
    assert txn_receipt is not None

    final_value = call(contract=string_contract,
                       contract_function='getValue')
    assert to_bytes(text=final_value) == to_bytes(text="ÄLÄMÖLÖ")

    txn = web3.eth.getTransaction(txn_hash)
    assert txn['gas'] == 200000


def test_auto_gas_computation_when_transacting(web3,
                                               STRING_CONTRACT,
                                               skip_if_testrpc,
                                               wait_for_block,
                                               call,
                                               transact):
    skip_if_testrpc(web3)

    wait_for_block(web3)

    StringContract = web3.eth.contract(**STRING_CONTRACT)

    deploy_txn = StringContract.constructor("Caqalai").transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn, 30)
    assert deploy_receipt is not None
    string_contract = StringContract(address=deploy_receipt['contractAddress'])

    gas_estimate = string_contract.functions.setValue(to_bytes(text="ÄLÄMÖLÖ")).estimateGas()

    # eth_abi will pass as raw bytes, no encoding
    # unless we encode ourselves
    txn_hash = transact(contract=string_contract,
                        contract_function="setValue",
                        func_args=[to_bytes(text="ÄLÄMÖLÖ")])
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash, 30)
    assert txn_receipt is not None

    final_value = call(contract=string_contract,
                       contract_function='getValue')
    assert to_bytes(text=final_value) == to_bytes(text="ÄLÄMÖLÖ")

    txn = web3.eth.getTransaction(txn_hash)
    assert txn['gas'] == gas_estimate + 100000


def test_fallback_transacting_with_contract(web3, fallback_function_contract, call):
    initial_value = call(contract=fallback_function_contract,
                         contract_function='getData')
    txn_hash = fallback_function_contract.fallback.transact()
    txn_receipt = web3.eth.waitForTransactionReceipt(txn_hash)
    assert txn_receipt is not None

    final_value = call(contract=fallback_function_contract,
                       contract_function='getData')

    assert final_value - initial_value == 1



import pytest
from unittest.mock import (
    Mock,
    patch,
)

from requests.exceptions import (
    ConnectionError,
    HTTPError,
    Timeout,
    TooManyRedirects,
)

import web3
from web3.middleware.exception_retry_request import (
    check_if_retry_on_failure,
    exception_retry_middleware,
)
from web3.providers import (
    HTTPProvider,
    IPCProvider,
)


@pytest.fixture
def exception_retry_request_setup():
    web3 = Mock()
    provider = HTTPProvider()
    errors = (ConnectionError, HTTPError, Timeout, TooManyRedirects)
    setup = exception_retry_middleware(provider.make_request, web3, errors, 5)
    setup.web3 = web3
    return setup


def test_check_if_retry_on_failure_false():
    methods = ['eth_sendTransaction', 'personal_signAndSendTransaction', 'personal_sendTRansaction']

    for method in methods:
        assert not check_if_retry_on_failure(method)


def test_check_if_retry_on_failure_true():
    method = 'eth_getBalance'
    assert check_if_retry_on_failure(method)


@patch('web3.providers.rpc.make_post_request', side_effect=ConnectionError)
def test_check_send_transaction_called_once(make_post_request_mock, exception_retry_request_setup):
    method = 'eth_sendTransaction'
    params = [{
        'to': '0x0',
        'value': 1,
    }]

    with pytest.raises(ConnectionError):
        exception_retry_request_setup(method, params)
    assert make_post_request_mock.call_count == 1


@patch('web3.providers.rpc.make_post_request', side_effect=ConnectionError)
def test_valid_method_retried(make_post_request_mock, exception_retry_request_setup):
    method = 'eth_getBalance'
    params = []

    with pytest.raises(ConnectionError):
        exception_retry_request_setup(method, params)
    assert make_post_request_mock.call_count == 5


def test_is_strictly_default_http_middleware():
    web3 = HTTPProvider()
    assert 'http_retry_request' in web3.middlewares

    web3 = IPCProvider()
    assert 'http_retry_request' not in web3.middlewares


@patch('web3.providers.rpc.make_post_request', side_effect=ConnectionError)
def test_check_with_all_middlewares(make_post_request_mock):
    provider = HTTPProvider()
    w3 = web3.Web3(provider)
    with pytest.raises(ConnectionError):
        w3.eth.blockNumber()
    assert make_post_request_mock.call_count == 5



import pytest
import random

from flaky import (
    flaky,
)

from web3.utils.threads import (
    Timeout,
)


@pytest.mark.skip(reason="fixture 'web3_empty' not found")
@flaky(max_runs=3)
def test_sync_filter_against_pending_transactions(web3_empty,
                                                  wait_for_transaction,
                                                  skip_if_testrpc
                                                  ):
    web3 = web3_empty
    skip_if_testrpc(web3)

    txn_filter = web3.eth.filter("pending")

    txn_1_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 12345,
    })
    txn_2_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 54321,
    })

    wait_for_transaction(web3, txn_1_hash)
    wait_for_transaction(web3, txn_2_hash)

    with Timeout(5) as timeout:
        while not txn_filter.get_new_entries():
            timeout.sleep(random.random())

    seen_txns = txn_filter.get_new_entries()

    assert txn_1_hash in seen_txns
    assert txn_2_hash in seen_txns


@pytest.mark.skip(reason="fixture 'web3_empty' not found")
@flaky(max_runs=3)
def test_async_filter_against_pending_transactions(web3_empty,
                                                   wait_for_transaction,
                                                   skip_if_testrpc
                                                   ):
    web3 = web3_empty
    skip_if_testrpc(web3)

    seen_txns = []
    txn_filter = web3.eth.filter("pending")
    txn_filter.watch(seen_txns.append)

    txn_1_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 12345,
    })
    txn_2_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': '0xd3CdA913deB6f67967B99D67aCDFa1712C293601',
        'value': 54321,
    })

    wait_for_transaction(web3, txn_1_hash)
    wait_for_transaction(web3, txn_2_hash)

    with Timeout(5) as timeout:
        while not seen_txns:
            timeout.sleep(random.random())

    txn_filter.stop_watching(30)

    assert txn_1_hash in seen_txns
    assert txn_2_hash in seen_txns



import json
import os
import pytest
import shutil
import subprocess

from eth_utils import (
    is_checksum_address,
    is_dict,
    to_text,
)

from web3.utils.toolz import (
    assoc,
)

from .utils import (
    kill_proc_gracefully,
)

KEYFILE_PW = 'web3py-test'


GETH_16_FIXTURE = {
    'datadir': 'geth-16-datadir-fixture',
    'block_hash_with_log': '0x5d84bd72195aacbbf6f3ed66be7a16495ed470cbc3e4764c69e4be75ab084148',
    'block_with_txn_hash': '0x4000549a8a573ed2e436de3a9014fdf71922f59aa11753870baa2ad03a32ebfc',
    'emitter_address': '0x4aA591a07989b4F810E2F5cE97e769D60710f168',
    'emitter_deploy_txn_hash': '0x1f676a3d88a8eb3210df677f3dca96edd78b646f8dcecab82d186d7394c8ab6c',
    'empty_block_hash': '0xd09336bcc6164d8d958914f7800356a3bb0cf05f98e20aefc00ce23d9ca62d2d',
    'keyfile_pw': 'web3py-test',
    'math_address': '0xd794C821fCCFF5D96F5Db44af7e29977630A9dc2',
    'math_deploy_txn_hash': '0xbefcf394f431fd983901d16c155da2d009da720b7b88cb9c7dce66f5d3ac44e7',
    'mined_txn_hash': '0x95110dd5943f513a1fd29767b48fe2178b973e99f5d73693d889081d7bdcd0c2',
    'raw_txn_account': '0x39EEed73fb1D3855E90Cbd42f348b3D7b340aAA6',
    'txn_hash_with_log': '0x2fd8dcd6ab1318245f8423df8e31f66f5d0fac2db34d7ab4a2a21a71037beae1',
}
GETH_17_FIXTURE = {
    'datadir': 'geth-17-datadir-fixture',
    'block_hash_with_log': '0x78a60c6b31c7af5e5ce87bad73b595dfe5b8715b161f4d3ded468ddcb14b5aeb',
    'block_with_txn_hash': '0x034faac7d0932774d9d837a97d55061a2dca9724c9779427a075f0a475aa3f43',
    'emitter_address': '0x4aA591a07989b4F810E2F5cE97e769D60710f168',
    'emitter_deploy_txn_hash': '0x1f676a3d88a8eb3210df677f3dca96edd78b646f8dcecab82d186d7394c8ab6c',
    'empty_block_hash': '0xc7a1b4c19f6c1d830a743f7a93a58bab129f4671f1eb1a82ae77e6643d733b9b',
    'keyfile_pw': 'web3py-test',
    'math_address': '0xd794C821fCCFF5D96F5Db44af7e29977630A9dc2',
    'math_deploy_txn_hash': '0xbefcf394f431fd983901d16c155da2d009da720b7b88cb9c7dce66f5d3ac44e7',
    'mined_txn_hash': '0x95110dd5943f513a1fd29767b48fe2178b973e99f5d73693d889081d7bdcd0c2',
    'raw_txn_account': '0x39EEed73fb1D3855E90Cbd42f348b3D7b340aAA6',
    'txn_hash_with_log': '0x2fd8dcd6ab1318245f8423df8e31f66f5d0fac2db34d7ab4a2a21a71037beae1',
}
GETH_181_DIRECTORY_NAME = 'geth-1.8.1-datadir-fixture'


@pytest.fixture(scope='module')
def geth_binary():
    from geth.install import (
        get_executable_path,
        install_geth,
    )

    if 'GETH_BINARY' in os.environ:
        return os.environ['GETH_BINARY']
    elif 'GETH_VERSION' in os.environ:
        geth_version = os.environ['GETH_VERSION']
        _geth_binary = get_executable_path(geth_version)
        if not os.path.exists(_geth_binary):
            install_geth(geth_version)
        assert os.path.exists(_geth_binary)
        return _geth_binary
    else:
        return 'geth'


def absolute_datadir(directory_name):
    return os.path.abspath(os.path.join(
        os.path.dirname(__file__),
        '..',
        directory_name,
    ))


def load_fixture_data(fixture_path):
    fixture_path = absolute_datadir(fixture_path)
    config_path = os.path.join(fixture_path, 'config.json')
    with open(config_path) as config_file:
        loaded_data = json.loads(config_file.read())
        return assoc(loaded_data, 'datadir', fixture_path)


@pytest.fixture(scope="module")
def geth_fixture_data(geth_binary):
    from geth import get_geth_version
    version = get_geth_version(geth_executable=os.path.expanduser(geth_binary))
    if version.major == 1:
        if version.minor == 6:
            return GETH_16_FIXTURE
        elif version.minor == 7:
            return GETH_17_FIXTURE
        elif version.minor == 8:
            return load_fixture_data(GETH_181_DIRECTORY_NAME)
    assert False, "Unsupported geth version"


@pytest.fixture(scope='module')
def datadir(tmpdir_factory, geth_fixture_data):
    fixture_datadir = absolute_datadir(geth_fixture_data['datadir'])
    base_dir = tmpdir_factory.mktemp('goethereum')
    tmp_datadir = os.path.join(str(base_dir), 'datadir')
    shutil.copytree(fixture_datadir, tmp_datadir)
    return tmp_datadir


@pytest.fixture(scope='module')
def genesis_file(datadir):
    genesis_file_path = os.path.join(datadir, 'genesis.json')
    return genesis_file_path


@pytest.fixture(scope='module')
def geth_process(geth_binary, datadir, genesis_file, geth_command_arguments):
    init_datadir_command = (
        geth_binary,
        '--datadir', str(datadir),
        'init',
        str(genesis_file),
    )
    subprocess.check_output(
        init_datadir_command,
        stdin=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    proc = subprocess.Popen(
        geth_command_arguments,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        bufsize=1,
    )
    try:
        yield proc
    finally:
        kill_proc_gracefully(proc)
        output, errors = proc.communicate()
        print(
            "Geth Process Exited:\n"
            "stdout:{0}\n\n"
            "stderr:{1}\n\n".format(
                to_text(output),
                to_text(errors),
            )
        )


@pytest.fixture(scope='module')
def coinbase(web3):
    return web3.eth.coinbase


@pytest.fixture(scope="module")
def math_contract_deploy_txn_hash(geth_fixture_data):
    return geth_fixture_data['math_deploy_txn_hash']


@pytest.fixture(scope="module")
def math_contract(web3, math_contract_factory, geth_fixture_data):
    return math_contract_factory(address=geth_fixture_data['math_address'])


@pytest.fixture(scope="module")
def math_contract_address(math_contract, address_conversion_func):
    return address_conversion_func(math_contract.address)


@pytest.fixture(scope="module")
def emitter_contract(web3, emitter_contract_factory, geth_fixture_data):
    return emitter_contract_factory(address=geth_fixture_data['emitter_address'])


@pytest.fixture(scope="module")
def emitter_contract_address(emitter_contract, address_conversion_func):
    return address_conversion_func(emitter_contract.address)


@pytest.fixture
def unlocked_account(web3, unlockable_account, unlockable_account_pw):
    web3.personal.unlockAccount(unlockable_account, unlockable_account_pw)
    yield unlockable_account
    web3.personal.lockAccount(unlockable_account)


@pytest.fixture(scope='module')
def unlockable_account_pw(geth_fixture_data):
    return geth_fixture_data['keyfile_pw']


@pytest.fixture(scope="module")
def unlockable_account(web3, coinbase):
    yield coinbase


@pytest.fixture()
def unlockable_account_dual_type(unlockable_account, address_conversion_func):
    return address_conversion_func(unlockable_account)


@pytest.yield_fixture
def unlocked_account_dual_type(web3, unlockable_account_dual_type, unlockable_account_pw):
    web3.personal.unlockAccount(unlockable_account_dual_type, unlockable_account_pw)
    yield unlockable_account_dual_type
    web3.personal.lockAccount(unlockable_account_dual_type)


@pytest.fixture(scope="module")
def funded_account_for_raw_txn(geth_fixture_data):
    account = geth_fixture_data['raw_txn_account']
    assert is_checksum_address(account)
    return account


@pytest.fixture(scope="module")
def empty_block(web3, geth_fixture_data):
    block = web3.eth.getBlock(geth_fixture_data['empty_block_hash'])
    assert is_dict(block)
    return block


@pytest.fixture(scope="module")
def block_with_txn(web3, geth_fixture_data):
    block = web3.eth.getBlock(geth_fixture_data['block_with_txn_hash'])
    assert is_dict(block)
    return block


@pytest.fixture(scope="module")
def mined_txn_hash(geth_fixture_data):
    return geth_fixture_data['mined_txn_hash']


@pytest.fixture(scope="module")
def block_with_txn_with_log(web3, geth_fixture_data):
    block = web3.eth.getBlock(geth_fixture_data['block_hash_with_log'])
    assert is_dict(block)
    return block


@pytest.fixture(scope="module")
def txn_hash_with_log(geth_fixture_data):
    return geth_fixture_data['txn_hash_with_log']



import pytest
from unittest.mock import (
    Mock,
)

from eth_utils import (
    to_bytes,
)

ABI = [{}]
ADDRESS = '0xd3CdA913deB6f67967B99D67aCDFa1712C293601'
BYTES_ADDRESS = to_bytes(hexstr=ADDRESS)
NON_CHECKSUM_ADDRESS = '0xd3cda913deb6f67967b99d67acdfa1712c293601'
INVALID_CHECKSUM_ADDRESS = '0xd3CDA913deB6f67967B99D67aCDFa1712C293601'


@pytest.mark.parametrize(
    'args,kwargs,expected',
    (
        ((ADDRESS,), {}, None),
        ((BYTES_ADDRESS,), {}, None),
        ((INVALID_CHECKSUM_ADDRESS,), {}, ValueError),
        ((NON_CHECKSUM_ADDRESS,), {}, ValueError),
        ((), {'address': ADDRESS}, None),
        ((), {'address': INVALID_CHECKSUM_ADDRESS}, ValueError),
        ((), {'address': NON_CHECKSUM_ADDRESS}, ValueError),
    )
)
def test_contract_address_validation(web3, args, kwargs, expected):
    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            web3.eth.contract(*args, **kwargs)
        return

    # run without errors
    web3.eth.contract(*args, **kwargs)


def test_set_contract_factory(web3):
    factoryClass = Mock()
    web3.eth.setContractFactory(factoryClass)
    web3.eth.contract(contract_name='myname')
    factoryClass.factory.assert_called_once_with(web3, contract_name='myname')



import pytest
import time
import warnings

from web3.providers.eth_tester import (
    EthereumTesterProvider,
)
from web3.utils.threads import (
    Timeout,
)
from web3.main import Web3


class PollDelayCounter:
    def __init__(self, initial_delay=0, max_delay=1, initial_step=0.01):
        self.initial_delay = initial_delay
        self.initial_step = initial_step
        self.max_delay = max_delay
        self.current_delay = initial_delay

    def __call__(self):
        delay = self.current_delay

        if self.current_delay == 0:
            self.current_delay += self.initial_step
        else:
            self.current_delay *= 2
            self.current_delay = min(self.current_delay, self.max_delay)

        return delay

    def reset(self):
        self.current_delay = self.initial_delay


@pytest.fixture()
def sleep_interval():
    return PollDelayCounter()


def is_all_testrpc_providers(providers):
    return all(
        isinstance(provider, EthereumTesterProvider)
        for provider
        in providers
    )


@pytest.fixture()
def skip_if_testrpc():

    def _skip_if_testrpc(web3):
        if is_all_testrpc_providers(web3.providers):
            pytest.skip()
    return _skip_if_testrpc


@pytest.fixture()
def wait_for_miner_start():
    def _wait_for_miner_start(web3, timeout=60):
        poll_delay_counter = PollDelayCounter()
        with Timeout(timeout) as timeout:
            while not web3.eth.mining or not web3.eth.hashrate:
                time.sleep(poll_delay_counter())
                timeout.check()
    return _wait_for_miner_start


@pytest.fixture()
def wait_for_block():
    def _wait_for_block(web3, block_number=1, timeout=None):
        if not timeout:
            timeout = (block_number - web3.eth.blockNumber) * 3
        poll_delay_counter = PollDelayCounter()
        with Timeout(timeout) as timeout:
            while True:
                if web3.eth.blockNumber >= block_number:
                    break
                web3.manager.request_blocking("evm_mine", [])
                timeout.sleep(poll_delay_counter())
    return _wait_for_block


@pytest.fixture()
def wait_for_transaction():
    def _wait_for_transaction(web3, txn_hash, timeout=120):
        poll_delay_counter = PollDelayCounter()
        with Timeout(timeout) as timeout:
            while True:
                txn_receipt = web3.eth.getTransactionReceipt(txn_hash)
                if txn_receipt is not None:
                    break
                time.sleep(poll_delay_counter())
                timeout.check()

        return txn_receipt
    return _wait_for_transaction


@pytest.fixture()
def web3():
    provider = EthereumTesterProvider()
    return Web3(provider)


@pytest.fixture(autouse=True)
def print_warnings():
    warnings.simplefilter('always')



import pytest
import time

from web3.utils.threads import (
    ThreadWithReturn,
    Timeout,
    spawn,
)


class CustomThreadClass(ThreadWithReturn):
    pass


def test_spawning_simple_thread():
    container = {
        'success': None,
    }

    def target_fn():
        container['success'] = True

    thread = spawn(target_fn)
    thread.join()

    assert container['success'] is True


def test_spawning_specific_thread_class():
    container = {
        'success': None,
    }

    def target_fn():
        container['success'] = True

    thread = spawn(target_fn, thread_class=CustomThreadClass)
    thread.join()

    assert isinstance(thread, CustomThreadClass)

    assert container['success'] is True


def test_thread_with_return_value():
    container = {
        'success': None,
    }

    def target_fn():
        container['success'] = True
        return 12345

    thread = spawn(target_fn)
    thread.join()

    assert container['success'] is True

    assert thread.get() == 12345


def test_inline_completion_before_timeout():
    timeout = Timeout(0.01)
    timeout.start()
    timeout.check()
    timeout.cancel()
    time.sleep(0.02)


def test_inline_timeout():
    timeout = Timeout(0.01)
    timeout.start()
    time.sleep(0.02)
    with pytest.raises(Timeout):
        timeout.check()


def test_contextmanager_completion_before_timeout():
    with Timeout(0.01) as timeout:
        timeout.check()
    time.sleep(0.02)


def test_contextmanager_timeout():
    with pytest.raises(Timeout):
        with Timeout(0.01) as timeout:
            time.sleep(0.02)
            timeout.check()


def test_with_custom_exception_type():
    timeout = Timeout(0.01, ValueError)
    timeout.start()
    time.sleep(0.02)
    with pytest.raises(ValueError):
        timeout.check()


def test_with_custom_exception_instance():
    exc = ValueError("an instance of an excepiton")
    timeout = Timeout(0.01, exc)
    timeout.start()
    time.sleep(0.02)
    with pytest.raises(ValueError) as err:
        timeout.check()

    assert err.value is exc



import hashlib
from json.decoder import (
    JSONDecodeError,
)
import os
import stat
import sys

from eth_utils import (
    to_tuple,
)
import requests
import toolz
from tqdm import tqdm

URI_QUERY_URL = "https://vanity-service.parity.io/parity-binaries"
BASE_BIN_PATH = "~/.parity-bin"
VERSION_STRINGS = {
    "v1.8.7": "1_8_7",
    "v1.8.8": "1_8_8",
    "v1.9.1": "1_9_1",
    "v1.10.4": "1_10_4",
    "v1.11.7": "1_11_7",
}
ARCHITECTURE = 'x86_64'
OS = os.getenv('PARITY_OS', 'debian')


@toolz.curry
def fill_default_request_params(version, os=OS, architecture=ARCHITECTURE):
    params = {'version': version, 'os': os, 'architecture': architecture}
    return params


@toolz.functoolz.memoize
def get_parity_release_json(**kwargs):
    try:
        return requests.get(
            URI_QUERY_URL,
            params=kwargs).json()
    except JSONDecodeError as json_err:
        raise EnvironmentError("Could not find parity release for %s with %r" % (
            URI_QUERY_URL,
            kwargs,
        )) from json_err


@to_tuple
def get_binary_uri(releases_json):
    for release in releases_json:
        binary_uri = ((uri.get('downloadUrl', None), uri.get('checksum', None))
                      for uri in release.get('files', tuple())
                      if uri.get('name') == 'parity')
        yield next(binary_uri)


def get_executable_path(version_string):
    identifier = VERSION_STRINGS[version_string]
    base_path = os.environ.get('PARITY_BASE_INSTALL_PATH', BASE_BIN_PATH)
    path = os.path.join(base_path, 'parity-{0}'.format(identifier))
    return os.path.expanduser(path)


def install_parity(version_string):
    if version_string not in VERSION_STRINGS.keys():
        raise ValueError("{0} is not an accepted version identifier.")

    path = get_executable_path(version_string)

    get_uri = toolz.functoolz.compose(
        get_binary_uri,
        get_parity_release_json)
    params = fill_default_request_params(version_string)
    uri, checksum = get_uri(**params)[0]

    if not os.path.exists(path):
        try:
            download_binary(path, uri, checksum)
        except AssertionError as exc:
            raise Exception("failed to download binary at uri %r with params %r" % (
                uri,
                params,
            )) from exc

    return path


def download_binary(path, uri, checksum):
    r = get_binary_stream(uri)
    total_size = int(r.headers.get('content-length', 0))
    os.makedirs(os.path.dirname(path), exist_ok=True)
    digest = hashlib.sha256()
    with open(path, 'wb') as f:
        with tqdm(total=total_size,
                  unit='B',
                  unit_scale=True,
                  unit_divisor=1024) as pbar:
            for data in r.iter_content(32 * 1024):
                f.write(data)
                pbar.update(len(data))
                digest.update(data)
    hex_digest = digest.hexdigest()
    assert hex_digest == checksum, "digest vs checksum: %r vs %r" % (hex_digest, checksum)
    chmod_plus_x(path)


def chmod_plus_x(executable_path):
    current_st = os.stat(executable_path)
    os.chmod(executable_path, current_st.st_mode | stat.S_IEXEC)


def get_binary_stream(uri):
    resp = requests.get(uri, stream=True)
    if resp.status_code == 200:
        return resp
    else:
        resp.raise_for_status()


if __name__ == '__main__':
    install_parity(sys.argv[1])




def test_miner_setEtherbase(web3_empty):
    web3 = web3_empty
    assert web3.eth.coinbase == web3.eth.accounts[0]
    new_account = web3.personal.newAccount('this-is-a-password')
    web3.miner.setEtherBase(new_account)
    assert web3.eth.coinbase == new_account



import contextlib
import os
import shutil
import signal
import socket
import subprocess
import tempfile
import time

from eth_utils import (
    is_checksum_address,
    to_text,
)

COINBASE = '0xdc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd'
COINBASE_PK = '0x58d23b55bc9cdce1f18c2500f40ff4ab7245df9a89505e9b1fa4851f623d241d'

KEYFILE_DATA = '{"address":"dc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd","crypto":{"cipher":"aes-128-ctr","ciphertext":"52e06bc9397ea9fa2f0dae8de2b3e8116e92a2ecca9ad5ff0061d1c449704e98","cipherparams":{"iv":"aa5d0a5370ef65395c1a6607af857124"},"kdf":"scrypt","kdfparams":{"dklen":32,"n":262144,"p":1,"r":8,"salt":"9fdf0764eb3645ffc184e166537f6fe70516bf0e34dc7311dea21f100f0c9263"},"mac":"4e0b51f42b865c15c485f4faefdd1f01a38637e5247f8c75ffe6a8c0eba856f6"},"id":"5a6124e0-10f1-4c1c-ae3e-d903eacb740a","version":3}'  # noqa: E501

KEYFILE_PW = 'web3py-test'
KEYFILE_FILENAME = 'UTC--2017-08-24T19-42-47.517572178Z--dc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd'  # noqa: E501

RAW_TXN_ACCOUNT = '0x39EEed73fb1D3855E90Cbd42f348b3D7b340aAA6'

UNLOCKABLE_PRIVATE_KEY = '0x392f63a79b1ff8774845f3fa69de4a13800a59e7083f5187f1558f0797ad0f01'
UNLOCKABLE_ACCOUNT = '0x12efdc31b1a8fa1a1e756dfd8a1601055c971e13'
UNLOCKABLE_ACCOUNT_PW = KEYFILE_PW

GENESIS_DATA = {
    "config": {
        "chainId": 1337,
        "homesteadBlock": 0,
        "eip150Block": 0,
        "eip155Block": 10,
        "eip158Block": 10,
        "eip160Block": 10
    },
    "nonce": "0x0000000000000042",
    "alloc": {
        COINBASE: {
            "balance": "1000000000000000000000000000"
        },
        UNLOCKABLE_ACCOUNT: {
            "balance": "1000000000000000000000000000"
        },
        RAW_TXN_ACCOUNT: {
            "balance": "1000000000000000000000000000"
        }
    },
    "timestamp": "0x00",
    "parentHash": "0x0000000000000000000000000000000000000000000000000000000000000000",
    "extraData": "0x3535353535353535353535353535353535353535353535353535353535353535",
    "gasLimit": "0x1000000",
    "difficulty": "0x10000",
    "mixhash": "0x0000000000000000000000000000000000000000000000000000000000000000",
    "coinbase": COINBASE
}


def ensure_path_exists(dir_path):
    """
    Make sure that a path exists
    """
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)
        return True
    return False


@contextlib.contextmanager
def tempdir():
    dir_path = tempfile.mkdtemp()
    try:
        yield dir_path
    finally:
        shutil.rmtree(dir_path)


def get_open_port():
    sock = socket.socket()
    sock.bind(('127.0.0.1', 0))
    port = sock.getsockname()[1]
    sock.close()
    return str(port)


def get_geth_binary():
    from geth.install import (
        get_executable_path,
        install_geth,
    )

    if 'GETH_BINARY' in os.environ:
        return os.environ['GETH_BINARY']
    elif 'GETH_VERSION' in os.environ:
        geth_version = os.environ['GETH_VERSION']
        _geth_binary = get_executable_path(geth_version)
        if not os.path.exists(_geth_binary):
            install_geth(geth_version)
        assert os.path.exists(_geth_binary)
        return _geth_binary
    else:
        return 'geth'


def wait_for_popen(proc, timeout):
    start = time.time()
    while time.time() < start + timeout:
        if proc.poll() is None:
            time.sleep(0.01)
        else:
            break


def kill_proc_gracefully(proc):
    if proc.poll() is None:
        proc.send_signal(signal.SIGINT)
        wait_for_popen(proc, 13)

    if proc.poll() is None:
        proc.terminate()
        wait_for_popen(proc, 5)

    if proc.poll() is None:
        proc.kill()
        wait_for_popen(proc, 2)


def wait_for_socket(ipc_path, timeout=30):
    start = time.time()
    while time.time() < start + timeout:
        try:
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            sock.connect(ipc_path)
            sock.settimeout(timeout)
        except (FileNotFoundError, socket.error):
            time.sleep(0.01)
        else:
            break


@contextlib.contextmanager
def get_geth_process(geth_binary,
                     datadir,
                     genesis_file_path,
                     ipc_path,
                     port,
                     networkid,
                     skip_init=False):
    if not skip_init:
        init_datadir_command = (
            geth_binary,
            '--datadir', datadir,
            'init',
            genesis_file_path,
        )
        print(' '.join(init_datadir_command))
        subprocess.check_output(
            init_datadir_command,
            stdin=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

    run_geth_command = (
        geth_binary,
        '--datadir', datadir,
        '--ipcpath', ipc_path,
        '--nodiscover',
        '--port', port,
        '--networkid', networkid,
        '--etherbase', COINBASE[2:],
    )
    print(' '.join(run_geth_command))
    try:
        proc = get_process(run_geth_command)
        yield proc
    finally:
        kill_proc_gracefully(proc)
        output, errors = proc.communicate()
        print(
            "Geth Process Exited:\n"
            "stdout:{0}\n\n"
            "stderr:{1}\n\n".format(
                to_text(output),
                to_text(errors),
            )
        )


def get_process(run_command):
    proc = subprocess.Popen(
        run_command,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        bufsize=1,
    )
    return proc


def mine_block(web3):
    origin_block_number = web3.eth.blockNumber

    start_time = time.time()
    web3.miner.start(1)
    while time.time() < start_time + 120:
        block_number = web3.eth.blockNumber
        if block_number > origin_block_number:
            web3.miner.stop()
            return block_number
        else:
            time.sleep(0.1)
    else:
        raise ValueError("No block mined during wait period")


def mine_transaction_hash(web3, txn_hash):
    start_time = time.time()
    web3.miner.start(1)
    while time.time() < start_time + 120:
        receipt = web3.eth.getTransactionReceipt(txn_hash)
        if receipt is not None:
            web3.miner.stop()
            return receipt
        else:
            time.sleep(0.1)
    else:
        raise ValueError("Math contract deploy transaction not mined during wait period")


def deploy_contract(web3, name, factory):
    web3.personal.unlockAccount(web3.eth.coinbase, KEYFILE_PW)
    deploy_txn_hash = factory.constructor().transact({'from': web3.eth.coinbase})
    print('{0}_CONTRACT_DEPLOY_HASH: '.format(name.upper()), deploy_txn_hash)
    deploy_receipt = mine_transaction_hash(web3, deploy_txn_hash)
    print('{0}_CONTRACT_DEPLOY_TRANSACTION_MINED'.format(name.upper()))
    contract_address = deploy_receipt['contractAddress']
    assert is_checksum_address(contract_address)
    print('{0}_CONTRACT_ADDRESS:'.format(name.upper()), contract_address)
    return deploy_receipt



import pytest

from web3.utils.datatypes import (
    PropertyCheckingFactory,
)


class InheritedBaseClass:
    arg0 = None
    arg1 = None


class BaseClass(InheritedBaseClass):
    arg2 = None
    arg3 = None


def test_property_checking_metaclass_attribute_error():
    # Test proper attribute checking, arg from both bases.
    namespace = {'arg2': True, 'arg0': True, 'arg4': True}
    with pytest.raises(AttributeError):
        PropertyCheckingFactory('class_name', (BaseClass,), namespace)

    # Test proper attribute checking, only absent arg.
    namespace = {'arg4': True}
    with pytest.raises(AttributeError):
        PropertyCheckingFactory('class_name', (BaseClass,), namespace)


def test_property_checking_metaclass_inherited_attributes():
    from_inherited_namespace = {'arg0': True, 'arg1': True}
    PropertyCheckingFactory('class_name', (BaseClass,), from_inherited_namespace)

    from_base_namespace = {'arg2': True, 'arg3': True}
    PropertyCheckingFactory('class_name', (BaseClass,), from_base_namespace)



def test_shh_post(web3, skip_if_testrpc):
    skip_if_testrpc(web3)
    receiver_pub = web3.shh.getPublicKey(web3.shh.newKeyPair())
    assert web3.shh.post({
        "topic": "0x12345678",
        "powTarget": 2.5,
        "powTime": 2,
        "payload": web3.toHex(text="testing shh on web3.py"),
        "pubKey": receiver_pub,
    })



import functools
import pytest

from eth_tester import (
    EthereumTester,
)
from eth_utils import (
    is_checksum_address,
    is_dict,
)

from web3 import Web3
from web3.providers.eth_tester import (
    EthereumTesterProvider,
)
from web3.utils.module_testing import (
    EthModuleTest,
    NetModuleTest,
    PersonalModuleTest,
    VersionModuleTest,
    Web3ModuleTest,
)
from web3.utils.module_testing.emitter_contract import (
    EMITTER_ENUM,
)

pytestmark = pytest.mark.filterwarnings("ignore:implicit cast from 'char *'")


@pytest.fixture(scope="module")
def eth_tester():
    _eth_tester = EthereumTester()
    return _eth_tester


@pytest.fixture(scope="module")
def eth_tester_provider(eth_tester):
    provider = EthereumTesterProvider(eth_tester)
    return provider


@pytest.fixture(scope="module")
def web3(eth_tester_provider):
    _web3 = Web3(eth_tester_provider)
    return _web3


#
# Math Contract Setup
#
@pytest.fixture(scope="module")
def math_contract_deploy_txn_hash(web3, math_contract_factory):
    deploy_txn_hash = math_contract_factory.constructor().transact({'from': web3.eth.coinbase})
    return deploy_txn_hash


@pytest.fixture(scope="module")
def math_contract(web3, math_contract_factory, math_contract_deploy_txn_hash):
    deploy_receipt = web3.eth.waitForTransactionReceipt(math_contract_deploy_txn_hash)
    assert is_dict(deploy_receipt)
    contract_address = deploy_receipt['contractAddress']
    assert is_checksum_address(contract_address)
    return math_contract_factory(contract_address)


@pytest.fixture(scope="module")
def math_contract_address(math_contract, address_conversion_func):
    return address_conversion_func(math_contract.address)

#
# Emitter Contract Setup
#


@pytest.fixture(scope="module")
def emitter_contract_deploy_txn_hash(web3, emitter_contract_factory):
    deploy_txn_hash = emitter_contract_factory.constructor().transact({'from': web3.eth.coinbase})
    return deploy_txn_hash


@pytest.fixture(scope="module")
def emitter_contract(web3, emitter_contract_factory, emitter_contract_deploy_txn_hash):
    deploy_receipt = web3.eth.waitForTransactionReceipt(emitter_contract_deploy_txn_hash)
    assert is_dict(deploy_receipt)
    contract_address = deploy_receipt['contractAddress']
    assert is_checksum_address(contract_address)
    return emitter_contract_factory(contract_address)


@pytest.fixture(scope="module")
def emitter_contract_address(emitter_contract, address_conversion_func):
    return address_conversion_func(emitter_contract.address)


@pytest.fixture(scope="module")
def empty_block(web3):
    web3.testing.mine()
    block = web3.eth.getBlock("latest")
    assert not block['transactions']
    return block


@pytest.fixture(scope="module")
def block_with_txn(web3):
    txn_hash = web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': web3.eth.coinbase,
        'value': 1,
        'gas': 21000,
        'gas_price': 1,
    })
    txn = web3.eth.getTransaction(txn_hash)
    block = web3.eth.getBlock(txn['blockNumber'])
    return block


@pytest.fixture(scope="module")
def mined_txn_hash(block_with_txn):
    return block_with_txn['transactions'][0]


@pytest.fixture(scope="module")
def block_with_txn_with_log(web3, emitter_contract):
    txn_hash = emitter_contract.functions.logDouble(
        which=EMITTER_ENUM['LogDoubleWithIndex'], arg0=12345, arg1=54321,
    ).transact({
        'from': web3.eth.coinbase,
    })
    txn = web3.eth.getTransaction(txn_hash)
    block = web3.eth.getBlock(txn['blockNumber'])
    return block


@pytest.fixture(scope="module")
def txn_hash_with_log(block_with_txn_with_log):
    return block_with_txn_with_log['transactions'][0]


UNLOCKABLE_PRIVATE_KEY = '0x392f63a79b1ff8774845f3fa69de4a13800a59e7083f5187f1558f0797ad0f01'


@pytest.fixture(scope='module')
def unlockable_account_pw(web3):
    return 'web3-testing'


@pytest.fixture(scope='module')
def unlockable_account(web3, unlockable_account_pw):
    account = web3.personal.importRawKey(UNLOCKABLE_PRIVATE_KEY, unlockable_account_pw)
    web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': account,
        'value': web3.toWei(10, 'ether'),
    })
    yield account


@pytest.fixture
def unlocked_account(web3, unlockable_account, unlockable_account_pw):
    web3.personal.unlockAccount(unlockable_account, unlockable_account_pw)
    yield unlockable_account
    web3.personal.lockAccount(unlockable_account)


@pytest.fixture()
def unlockable_account_dual_type(unlockable_account, address_conversion_func):
    return address_conversion_func(unlockable_account)


@pytest.fixture
def unlocked_account_dual_type(web3, unlockable_account_dual_type, unlockable_account_pw):
    web3.personal.unlockAccount(unlockable_account_dual_type, unlockable_account_pw)
    yield unlockable_account_dual_type
    web3.personal.lockAccount(unlockable_account_dual_type)


@pytest.fixture(scope="module")
def funded_account_for_raw_txn(web3):
    account = '0x39EEed73fb1D3855E90Cbd42f348b3D7b340aAA6'
    web3.eth.sendTransaction({
        'from': web3.eth.coinbase,
        'to': account,
        'value': web3.toWei(10, 'ether'),
        'gas': 21000,
        'gas_price': 1,
    })
    return account


class TestEthereumTesterWeb3Module(Web3ModuleTest):
    def _check_web3_clientVersion(self, client_version):
        assert client_version.startswith('EthereumTester/')


def not_implemented(method, exc_type=NotImplementedError):
    @functools.wraps(method)
    def inner(*args, **kwargs):
        with pytest.raises(exc_type):
            method(*args, **kwargs)
    return inner


def disable_auto_mine(func):
    @functools.wraps(func)
    def func_wrapper(self, eth_tester, *args, **kwargs):
        snapshot = eth_tester.take_snapshot()
        eth_tester.disable_auto_mine_transactions()
        try:
            func(self, eth_tester, *args, **kwargs)
        finally:
            eth_tester.enable_auto_mine_transactions()
            eth_tester.mine_block()
            eth_tester.revert_to_snapshot(snapshot)
    return func_wrapper


class TestEthereumTesterEthModule(EthModuleTest):
    test_eth_sign = not_implemented(EthModuleTest.test_eth_sign, ValueError)

    @disable_auto_mine
    def test_eth_getTransactionReceipt_unmined(self, eth_tester, web3, unlocked_account):
        super().test_eth_getTransactionReceipt_unmined(web3, unlocked_account)

    @disable_auto_mine
    def test_eth_replaceTransaction(self, eth_tester, web3, unlocked_account):
        super().test_eth_replaceTransaction(web3, unlocked_account)

    @disable_auto_mine
    def test_eth_replaceTransaction_incorrect_nonce(self, eth_tester, web3, unlocked_account):
        super().test_eth_replaceTransaction_incorrect_nonce(web3, unlocked_account)

    @disable_auto_mine
    def test_eth_replaceTransaction_gas_price_too_low(self, eth_tester, web3, unlocked_account):
        super().test_eth_replaceTransaction_gas_price_too_low(web3, unlocked_account)

    @disable_auto_mine
    def test_eth_replaceTransaction_gas_price_defaulting_minimum(self,
                                                                 eth_tester,
                                                                 web3,
                                                                 unlocked_account):
        super().test_eth_replaceTransaction_gas_price_defaulting_minimum(web3, unlocked_account)

    @disable_auto_mine
    def test_eth_replaceTransaction_gas_price_defaulting_strategy_higher(self,
                                                                         eth_tester,
                                                                         web3,
                                                                         unlocked_account):
        super().test_eth_replaceTransaction_gas_price_defaulting_strategy_higher(
            web3, unlocked_account
        )

    @disable_auto_mine
    def test_eth_replaceTransaction_gas_price_defaulting_strategy_lower(self,
                                                                        eth_tester,
                                                                        web3,
                                                                        unlocked_account):
        super().test_eth_replaceTransaction_gas_price_defaulting_strategy_lower(
            web3, unlocked_account
        )

    @disable_auto_mine
    def test_eth_modifyTransaction(self, eth_tester, web3, unlocked_account):
        super().test_eth_modifyTransaction(web3, unlocked_account)

    @disable_auto_mine
    def test_eth_call_old_contract_state(self, eth_tester, web3, math_contract, unlocked_account):
        # For now, ethereum tester cannot give call results in the pending block.
        # Once that feature is added, then delete the except/else blocks.
        try:
            super().test_eth_call_old_contract_state(web3, math_contract, unlocked_account)
        except AssertionError as err:
            if str(err) == "pending call result was 0 instead of 1":
                pass
            else:
                raise err
        else:
            raise AssertionError("eth-tester was unexpectedly able to give the pending call result")

    def test_eth_getStorageAt(self, web3, emitter_contract_address):
        pytest.xfail('json-rpc method is not implemented on eth-tester')
        super().test_eth_getStorageAt(web3, emitter_contract_address)


class TestEthereumTesterVersionModule(VersionModuleTest):
    pass


class TestEthereumTesterNetModule(NetModuleTest):
    pass


class TestEthereumTesterPersonalModule(PersonalModuleTest):
    test_personal_sign_and_ecrecover = not_implemented(
        PersonalModuleTest.test_personal_sign_and_ecrecover,
        ValueError,
    )



import pytest

from web3.utils.events import (
    construct_event_topic_set,
)

EVENT_1_ABI = {
    "anonymous": False,
    "inputs": [
        {"indexed": False, "name": "arg0", "type": "uint256"},
        {"indexed": True, "name": "arg1", "type": "uint256"},
        {"indexed": True, "name": "arg2", "type": "uint256"},
        {"indexed": False, "name": "arg3", "type": "uint256"},
        {"indexed": True, "name": "arg4", "type": "uint256"},
        {"indexed": False, "name": "arg5", "type": "uint256"},
    ],
    "name": "Event_1",
    "type": "event",
}
EVENT_1_TOPIC = '0xa7144ed450ecab4a6283d3b1e290ff6c889232d922b84d88203eb7619222fb32'


def hex_and_pad(i):
    unpadded_hex_value = hex(i).rstrip('L')
    return '0x' + unpadded_hex_value[2:].zfill(64)


@pytest.mark.parametrize(
    'event_abi,arguments,expected',
    (
        (
            EVENT_1_ABI,
            {},
            [EVENT_1_TOPIC],
        ),
        (
            EVENT_1_ABI,
            {'arg0': 1},
            [EVENT_1_TOPIC],
        ),
        (
            EVENT_1_ABI,
            {'arg0': 1, 'arg3': [1, 2]},
            [EVENT_1_TOPIC],
        ),
        (
            EVENT_1_ABI,
            {'arg1': 1},
            [
                EVENT_1_TOPIC, hex_and_pad(1)
            ],
        ),
        (
            EVENT_1_ABI,
            {'arg1': [1, 2]},
            [
                EVENT_1_TOPIC, [hex_and_pad(1), hex_and_pad(2)],
            ],
        ),
        (
            EVENT_1_ABI,
            {'arg1': [1], 'arg2': [2]},
            [
                EVENT_1_TOPIC, hex_and_pad(1), hex_and_pad(2),
            ],
        ),
        (
            EVENT_1_ABI,
            {'arg1': [1, 3], 'arg2': [2, 4]},
            [
                EVENT_1_TOPIC,
                [hex_and_pad(1), hex_and_pad(3)],
                [hex_and_pad(2), hex_and_pad(4)]
            ],
        ),
    )
)
def test_construct_event_topics(event_abi, arguments, expected):
    actual = construct_event_topic_set(event_abi, arguments)
    assert actual == expected



from ast import (
    literal_eval,
)
import datetime
import pytest
from unittest.mock import (
    Mock,
)

from eth_utils import (
    is_hex,
)
from hypothesis import (
    example,
    given,
    strategies as st,
)

from web3.providers import (
    JSONBaseProvider,
)
from web3.utils.encoding import (
    FriendlyJsonSerde as FriendlyJson,
    hex_encode_abi_type,
    hexstr_if_str,
    text_if_str,
    to_hex,
    to_int,
)
from web3.utils.hypothesis import (
    hexstr_strategy,
)


@pytest.mark.parametrize(
    "value,expected",
    [
        (1, '0x1'),
        (15, '0xf'),
        (-1, '-0x1'),
        (-15, '-0xf'),
        (0, '0x0'),
        (-0, '0x0'),
    ]
)
def test_to_hex(value, expected):
    assert to_hex(value) == expected


@given(value=st.integers(min_value=-1 * 2**255 + 1, max_value=2**256 - 1))
def test_conversion_round_trip(value):
    intermediate_value = to_hex(value)
    result_value = to_int(hexstr=intermediate_value)
    error_msg = "Expected: {0!r}, Result: {1!r}, Intermediate: {2!r}".format(
        value,
        result_value,
        intermediate_value,
    )
    assert result_value == value, error_msg


def test_bytes_that_start_with_0x():
    sneaky_bytes = b'0x\xde\xad'
    assert to_hex(sneaky_bytes) == '0x3078dead'


@pytest.mark.parametrize(
    "abi_type,value,expected",
    [
        ('bool', True, "0x01"),
        ('bool', False, "0x00"),
        ('uint16', 8, "0x0008"),
        ('int16', 8, "0x0008"),
        ('int16', -8, "0xfff8"),
        (
            'address',
            "0x00360d2b7D240Ec0643B6D819ba81A09e40E5bCd",
            "0x00360d2b7D240Ec0643B6D819ba81A09e40E5bCd"
        ),
        ("bytes2", b"T\x02", "0x5402"),
        ("bytes3", b"T\x02", "0x5402"),
        ("bytes", '0x5402', "0x5402"),
        ("bytes", '5402', TypeError),
        ("string", "testing a string!", "0x74657374696e67206120737472696e6721"),
        ("strings", "bad abi!", ValueError),
        ("bool[", True, ValueError),
        ("bool", "string", TypeError),
        ("uint24", -20, TypeError),
    ]
)
def test_hex_encode_abi_type(abi_type, value, expected):

    if isinstance(expected, type) and issubclass(expected, Exception):
        with pytest.raises(expected):
            hex_encode_abi_type(abi_type, value)
        return

    actual = hex_encode_abi_type(abi_type, value)
    assert actual == expected


@given(st.one_of(st.integers(), st.booleans(), st.binary()))
@example(b'')
def test_hexstr_if_str_passthrough(val):
    to_type = Mock(return_value='zoot')
    assert hexstr_if_str(to_type, val) == 'zoot'
    assert to_type.call_args == ((val, ), {'hexstr': None})


def test_hexstr_if_str_curried():
    converter = hexstr_if_str(to_hex)
    assert converter(255) == '0xff'


@given(hexstr_strategy())
@example('0x')
@example('0')
def test_hexstr_if_str_on_valid_hex(val):
    to_type = Mock(return_value='zoot')
    assert hexstr_if_str(to_type, val) == 'zoot'
    assert to_type.call_args == ((None, ), {'hexstr': val})


@given(st.text())
def test_hexstr_if_str_on_invalid_hex(val):
    try:
        is_hexstr = (is_hex(val) or val == '')
    except ValueError:
        is_hexstr = False

    if not is_hexstr:
        with pytest.raises(ValueError):
            hexstr_if_str(Mock(), val)


@given(st.one_of(st.integers(), st.booleans(), st.binary()))
@example(b'')
def test_text_if_str_passthrough(val):
    to_type = Mock(return_value='zoot')
    assert text_if_str(to_type, val) == 'zoot'
    assert to_type.call_args == ((val, ), {'text': None})


@given(st.text())
@example('0xa1')  # valid hexadecimal is still interpreted as unicode characters
def test_text_if_str_on_text(val):
    to_type = Mock(return_value='zoot')
    assert text_if_str(to_type, val) == 'zoot'
    assert to_type.call_args == ((None, ), {'text': val})


@pytest.mark.parametrize(
    "py_obj, exc_type, expected",
    (
        (
            {
                'date': [
                    datetime.datetime(2018, 5, 10, 1, 5, 10).isoformat(),
                    datetime.datetime(2018, 5, 10, 1, 5, 10).isoformat(),
                ],
                'other_date': datetime.datetime(2018, 5, 10, 1, 5, 10).date().isoformat(),
            },
            None,
            '{"date": ["2018-05-10T01:05:10", "2018-05-10T01:05:10"], "other_date": "2018-05-10"}',
        ),
        (
            {
                'date': [datetime.datetime.utcnow(), datetime.datetime.now()],
                'other_date': datetime.datetime.utcnow().date(),
            },
            TypeError,
            "Could not encode to JSON: .*'other_date'.*is not JSON serializable",
        ),
    ),
)
def test_friendly_json_encode(py_obj, exc_type, expected):
    if exc_type is None:
        assert literal_eval(FriendlyJson().json_encode(py_obj)) == literal_eval(expected)
    else:
        with pytest.raises(exc_type, match=expected):
            FriendlyJson().json_encode(py_obj)


@pytest.mark.parametrize(
    "json_str, expected",
    (
        (
            '{"date": ["2018-05-10T01:05:10", "2018-05-10T01:05:10"],"other_date": "2018-05-10"}',
            dict,
        ),
    ),
)
def test_friendly_json_decode(json_str, expected):
    assert isinstance(FriendlyJson().json_decode(json_str), expected)


@pytest.mark.parametrize(
    "rpc_response, expected",
    (
        (
            '{"jsonrpc": "2.0", "method": "test_method", "params": [], "id": 1}',
            {"jsonrpc": "2.0", "method": "test_method", "params": [], "id": 1},
        ),
    ),
)
def test_decode_rpc_response(rpc_response, expected):
    assert JSONBaseProvider().decode_rpc_response(rpc_response.encode('utf8')) == expected


@pytest.mark.parametrize(
    "rpc_kwargs, exc_type, expected",
    (
        (
            {'id': 1, 'method': 'test', 'params': [], "jsonrpc": "2.0"},
            None,
            '{"id": 0, "method": "test", "params": [], "jsonrpc": "2.0",}',
        ),
        (
            {'id': 0, 'method': 'test', 'params': [datetime.datetime(2018, 5, 10, 1, 5, 10)]},
            TypeError,
            r"Could not encode to JSON: .*'params'.* is not JSON serializable",
        ),
    ),
)
def test_encode_rpc_request(rpc_kwargs, exc_type, expected):
    if exc_type is None:
        res = JSONBaseProvider().encode_rpc_request(
            rpc_kwargs['method'],
            rpc_kwargs['params']
        )
        assert literal_eval(res.decode('utf8')) == literal_eval(expected)
    else:
        with pytest.raises(exc_type, match=expected):
            JSONBaseProvider().encode_rpc_request(
                rpc_kwargs['method'],
                rpc_kwargs['params'],
            )



import time

from web3.utils.decorators import (
    reject_recursive_repeats,
)
from web3.utils.threads import (
    spawn,
)


def test_reject_recursive_repeats_multithreaded():
    @reject_recursive_repeats
    def recurse(sleep_now):
        time.sleep(sleep_now)
        try:
            recurse(0.05)
            return True
        except ValueError:
            return False
    thd1 = spawn(recurse, 0)
    thd2 = spawn(recurse, 0.02)
    assert thd2.get() and thd1.get()



import pytest
import random

from eth_utils import (
    to_tuple,
)


@to_tuple
def deploy_contracts(web3, contract, wait_for_transaction):
    for i in range(25):
        tx_hash = contract.constructor().transact()
        wait_for_transaction(web3, tx_hash)
        yield web3.eth.getTransactionReceipt(tx_hash)['contractAddress']


def pad_with_transactions(w3):
    accounts = w3.eth.accounts
    for tx_count in range(random.randint(0, 10)):
        _from = accounts[random.randint(0, len(accounts) - 1)]
        _to = accounts[random.randint(0, len(accounts) - 1)]
        value = 50 + tx_count
        w3.eth.sendTransaction({'from': _from, 'to': _to, 'value': value})


def single_transaction(w3):
    accounts = w3.eth.accounts
    _from = accounts[random.randint(0, len(accounts) - 1)]
    _to = accounts[random.randint(0, len(accounts) - 1)]
    value = 50
    tx_hash = w3.eth.sendTransaction({'from': _from, 'to': _to, 'value': value})
    return tx_hash


def test_event_filter_new_events(
        web3,
        emitter,
        Emitter,
        wait_for_transaction,
        emitter_event_ids,
        create_filter):

    matching_transact = emitter.functions.logNoArgs(
        which=1).transact
    non_matching_transact = emitter.functions.logNoArgs(
        which=0).transact

    event_filter = emitter.events.LogNoArguments().createFilter(fromBlock='latest')

    expected_match_counter = 0

    while web3.eth.blockNumber < 50:
        is_match = bool(random.randint(0, 1))
        if is_match:
            expected_match_counter += 1
            matching_transact()
            pad_with_transactions(web3)
            continue
        non_matching_transact()
        pad_with_transactions(web3)

    assert len(event_filter.get_new_entries()) == expected_match_counter


@pytest.mark.xfail(reason="Suspected eth-tester bug")
def test_block_filter(web3):
    block_filter = web3.eth.filter("latest")

    while web3.eth.blockNumber < 50:
        pad_with_transactions(web3)

    assert len(block_filter.get_new_entries()) == web3.eth.blockNumber


def test_transaction_filter_with_mining(
        web3):

    transaction_filter = web3.eth.filter("pending")

    transaction_counter = 0

    while transaction_counter < 100:
        single_transaction(web3)
        transaction_counter += 1

    assert len(transaction_filter.get_new_entries()) == transaction_counter


@pytest.mark.xfail(reason="Suspected eth-tester bug")
def test_transaction_filter_without_mining(
        web3):

    web3.providers[0].ethereum_tester.auto_mine_transactions = False
    transaction_filter = web3.eth.filter("pending")

    transaction_counter = 0

    transact_once = single_transaction(web3)
    while transaction_counter < 100:
        next(transact_once)
        transaction_counter += 1

    assert len(transaction_filter.get_new_entries()) == transaction_counter


def test_event_filter_new_events_many_deployed_contracts(
        web3,
        emitter,
        Emitter,
        wait_for_transaction,
        emitter_event_ids,
        create_filter):

    matching_transact = emitter.functions.logNoArgs(
        which=1).transact

    deployed_contract_addresses = deploy_contracts(web3, Emitter, wait_for_transaction)

    def gen_non_matching_transact():
        while True:
            contract_address = deployed_contract_addresses[
                random.randint(0, len(deployed_contract_addresses) - 1)]
            yield web3.eth.contract(
                address=contract_address, abi=Emitter.abi).functions.logNoArgs(which=1).transact

    non_matching_transact = gen_non_matching_transact()

    event_filter = emitter.events.LogNoArguments().createFilter(fromBlock='latest')

    expected_match_counter = 0

    while web3.eth.blockNumber < 50:
        is_match = bool(random.randint(0, 1))
        if is_match:
            expected_match_counter += 1
            matching_transact()
            pad_with_transactions(web3)
            continue
        next(non_matching_transact)()
        pad_with_transactions(web3)

    assert len(event_filter.get_new_entries()) == expected_match_counter



def test_admin_addPeer(web3, skip_if_testrpc):
    skip_if_testrpc(web3)

    result = web3.admin.addPeer(
        'enode://44826a5d6a55f88a18298bca4773fca5749cdc3a5c9f308aa7d810e9b31123f3e7c5fba0b1d70aac5308426f47df2a128a6747040a3815cc7dd7167d03be320d@127.0.0.1:30304',  # noqa: E501
    )
    assert result is True



import random

from flaky import (
    flaky,
)

from web3.utils.threads import (
    Timeout,
)


@flaky(max_runs=3)
def test_miner_stop(web3_empty):
    web3 = web3_empty

    assert web3.eth.mining
    assert web3.miner.hashrate

    web3.miner.stop()

    with Timeout(60) as timeout:
        while web3.eth.mining or web3.eth.hashrate:
            timeout.sleep(random.random())
            timeout.check()

    assert not web3.eth.mining
    assert not web3.miner.hashrate



import time

from hexbytes import (
    HexBytes,
)


def test_shh_sync_filter(web3, skip_if_testrpc):
    skip_if_testrpc(web3)

    sender = web3.shh.newKeyPair()
    sender_pub = web3.shh.getPublicKey(sender)

    receiver = web3.shh.newKeyPair()
    receiver_pub = web3.shh.getPublicKey(receiver)

    topic = '0x13370000'
    payloads = [web3.toHex(text="test message :)"), web3.toHex(text="2nd test message")]

    shh_filter = web3.shh.newMessageFilter({
        'privateKeyID': receiver,
        'sig': sender_pub,
        'topics': [topic]
    })

    web3.shh.post({
        'sig': sender,
        'powTarget': 2.5,
        'powTime': 2,
        'payload': payloads[0],
        'pubKey': receiver_pub
    })
    time.sleep(1)

    web3.shh.post({
        'sig': sender,
        'powTarget': 2.5,
        'powTime': 2,
        'payload': payloads[1],
        'topic': topic,
        'pubKey': receiver_pub
    })
    time.sleep(1)

    received_messages = shh_filter.get_new_entries()
    assert len(received_messages) == 1

    message = received_messages[0]

    assert message["payload"] == HexBytes(payloads[1])
    assert message["topic"] == HexBytes(topic)


def test_shh_async_filter(web3, skip_if_testrpc):
    skip_if_testrpc(web3)
    received_messages = []

    sender = web3.shh.newKeyPair()
    sender_pub = web3.shh.getPublicKey(sender)

    receiver = web3.shh.newKeyPair()
    receiver_pub = web3.shh.getPublicKey(receiver)

    topic = '0x13370000'
    payloads = [web3.toHex(text="test message :)"), web3.toHex(text="2nd test message")]

    shh_filter = web3.shh.newMessageFilter({
        'privateKeyID': receiver,
        'sig': sender_pub,
        'topics': [topic]
    }, poll_interval=0.5)
    watcher = shh_filter.watch(received_messages.extend)

    web3.shh.post({
        'sig': sender,
        'powTarget': 2.5,
        'powTime': 2,
        'payload': payloads[0],
        'topic': topic,
        'pubKey': receiver_pub
    })
    time.sleep(1)

    web3.shh.post({
        'sig': sender,
        'powTarget': 2.5,
        'powTime': 2,
        'payload': payloads[1],
        'pubKey': receiver_pub
    })
    time.sleep(1)

    assert len(received_messages) == 1

    message = received_messages[0]

    assert message["payload"] == HexBytes(payloads[0])
    assert message["topic"] == HexBytes(topic)

    watcher.stop()


def test_shh_remove_filter(web3, skip_if_testrpc):
    skip_if_testrpc(web3)

    receiver = web3.shh.newKeyPair()
    receiver_pub = web3.shh.getPublicKey(receiver)

    payload = web3.toHex(text="test message :)")
    shh_filter = web3.shh.newMessageFilter({'privateKeyID': receiver})

    web3.shh.post({
        'powTarget': 2.5,
        'powTime': 2,
        'payload': payload,
        'pubKey': receiver_pub
    })
    time.sleep(1)

    message = shh_filter.get_new_entries()[0]
    assert message["payload"] == HexBytes(payload)

    assert web3.shh.deleteMessageFilter(shh_filter.filter_id)

    try:
        web3.shh.getMessages(shh_filter.filter_id)
        assert False
    except:
        assert True



import pytest

from eth_utils import (
    decode_hex,
)

TEST_ADDRESS = '0x16D9983245De15E7A9A73bC586E01FF6E08dE737'
EXPECTED_DATA_A = 1234
EXPECTED_DATA_B = (b'abcd\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
                   b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00')


def test_contract_constructor_abi_encoding_with_no_constructor_fn(MathContract, MATH_CODE):
    deploy_data = MathContract.constructor()._encode_data_in_transaction()
    assert deploy_data == MATH_CODE


def test_contract_constructor_gas_estimate_no_constructor(web3, MathContract):
    gas_estimate = MathContract.constructor().estimateGas()

    deploy_txn = MathContract.constructor().transact()
    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000


def test_contract_constructor_gas_estimate_with_constructor_without_arguments(
        web3,
        SimpleConstructorContract):
    gas_estimate = SimpleConstructorContract.constructor().estimateGas()

    deploy_txn = SimpleConstructorContract.constructor().transact()
    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000


@pytest.mark.parametrize(
    'constructor_args,constructor_kwargs',
    (
        ([1234, b'abcd'], {}),
        ([1234], {'b': b'abcd'}),
        ([], {'a': 1234, 'b': b'abcd'}),
        ([], {'b': b'abcd', 'a': 1234}),
    ),
)
def test_contract_constructor_gas_estimate_with_constructor_with_arguments(
        web3,
        WithConstructorArgumentsContract,
        constructor_args,
        constructor_kwargs):
    gas_estimate = WithConstructorArgumentsContract.constructor(
        *constructor_args, **constructor_kwargs).estimateGas()

    deploy_txn = WithConstructorArgumentsContract.constructor(
        *constructor_args, **constructor_kwargs).transact()
    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000


def test_contract_constructor_gas_estimate_with_constructor_with_address_argument(
        web3,
        WithConstructorAddressArgumentsContract,
        address_conversion_func):
    gas_estimate = WithConstructorAddressArgumentsContract.constructor(
        address_conversion_func("0x16D9983245De15E7A9A73bC586E01FF6E08dE737")).estimateGas()

    deploy_txn = WithConstructorAddressArgumentsContract.constructor(
        address_conversion_func("0x16D9983245De15E7A9A73bC586E01FF6E08dE737")).transact()
    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    gas_used = txn_receipt.get('gasUsed')

    assert abs(gas_estimate - gas_used) < 21000


def test_contract_constructor_transact_no_constructor(
        web3,
        MathContract,
        MATH_RUNTIME,
        address_conversion_func):
    deploy_txn = MathContract.constructor().transact()

    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert txn_receipt is not None

    assert txn_receipt['contractAddress']
    contract_address = address_conversion_func(txn_receipt['contractAddress'])

    blockchain_code = web3.eth.getCode(contract_address)
    assert blockchain_code == decode_hex(MATH_RUNTIME)


def test_contract_constructor_transact_with_constructor_without_arguments(
        web3,
        SimpleConstructorContract,
        SIMPLE_CONSTRUCTOR_RUNTIME,
        address_conversion_func):
    deploy_txn = SimpleConstructorContract.constructor().transact()

    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert txn_receipt is not None

    assert txn_receipt['contractAddress']
    contract_address = address_conversion_func(txn_receipt['contractAddress'])

    blockchain_code = web3.eth.getCode(contract_address)
    assert blockchain_code == decode_hex(SIMPLE_CONSTRUCTOR_RUNTIME)


@pytest.mark.parametrize(
    'constructor_args,constructor_kwargs, expected_a, expected_b',
    (
        ([1234, b'abcd'], {}, EXPECTED_DATA_A, EXPECTED_DATA_B),
        ([1234], {'b': b'abcd'}, EXPECTED_DATA_A, EXPECTED_DATA_B),
        ([], {'a': 1234, 'b': b'abcd'}, EXPECTED_DATA_A, EXPECTED_DATA_B),
        ([], {'b': b'abcd', 'a': 1234}, EXPECTED_DATA_A, EXPECTED_DATA_B),
    ),
)
def test_contract_constructor_transact_with_constructor_with_arguments(
        web3,
        WithConstructorArgumentsContract,
        WITH_CONSTRUCTOR_ARGUMENTS_RUNTIME,
        constructor_args,
        constructor_kwargs,
        expected_a,
        expected_b,
        address_conversion_func):
    deploy_txn = WithConstructorArgumentsContract.constructor(
        *constructor_args, **constructor_kwargs).transact()

    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert txn_receipt is not None

    assert txn_receipt['contractAddress']
    contract_address = address_conversion_func(txn_receipt['contractAddress'])

    blockchain_code = web3.eth.getCode(contract_address)
    assert blockchain_code == decode_hex(WITH_CONSTRUCTOR_ARGUMENTS_RUNTIME)
    assert expected_a == WithConstructorArgumentsContract(
        address=contract_address).functions.data_a().call()
    assert expected_b == WithConstructorArgumentsContract(
        address=contract_address).functions.data_b().call()


def test_contract_constructor_transact_with_constructor_with_address_arguments(
        web3,
        WithConstructorAddressArgumentsContract,
        WITH_CONSTRUCTOR_ADDRESS_RUNTIME,
        address_conversion_func):
    deploy_txn = WithConstructorAddressArgumentsContract.constructor(TEST_ADDRESS).transact()
    txn_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert txn_receipt is not None
    assert txn_receipt['contractAddress']
    contract_address = address_conversion_func(txn_receipt['contractAddress'])
    blockchain_code = web3.eth.getCode(contract_address)
    assert blockchain_code == decode_hex(WITH_CONSTRUCTOR_ADDRESS_RUNTIME)
    assert TEST_ADDRESS == WithConstructorAddressArgumentsContract(
        address=contract_address).functions.testAddr().call()


def test_contract_constructor_build_transaction_to_field_error(MathContract):
    with pytest.raises(ValueError):
        MathContract.constructor().buildTransaction({'to': '123'})


def test_contract_constructor_build_transaction_no_constructor(
        web3,
        MathContract,
        address_conversion_func):
    txn_hash = MathContract.constructor().transact(
        {'from': address_conversion_func(web3.eth.accounts[0])}
    )
    txn = web3.eth.getTransaction(txn_hash)
    nonce = web3.eth.getTransactionCount(web3.eth.coinbase)
    unsent_txn = MathContract.constructor().buildTransaction({'nonce': nonce})
    assert txn['data'] == unsent_txn['data']

    new_txn_hash = web3.eth.sendTransaction(unsent_txn)
    new_txn = web3.eth.getTransaction(new_txn_hash)
    assert new_txn['data'] == unsent_txn['data']
    assert new_txn['nonce'] == nonce


def test_contract_constructor_build_transaction_with_constructor_without_argument(
        web3,
        MathContract,
        address_conversion_func):
    txn_hash = MathContract.constructor().transact(
        {'from': address_conversion_func(web3.eth.accounts[0])}
    )
    txn = web3.eth.getTransaction(txn_hash)
    nonce = web3.eth.getTransactionCount(web3.eth.coinbase)
    unsent_txn = MathContract.constructor().buildTransaction({'nonce': nonce})
    assert txn['data'] == unsent_txn['data']

    new_txn_hash = web3.eth.sendTransaction(unsent_txn)
    new_txn = web3.eth.getTransaction(new_txn_hash)
    assert new_txn['data'] == unsent_txn['data']
    assert new_txn['nonce'] == nonce


@pytest.mark.parametrize(
    'constructor_args,constructor_kwargs',
    (
        ([1234, b'abcd'], {}),
        ([1234], {'b': b'abcd'}),
        ([], {'a': 1234, 'b': b'abcd'}),
        ([], {'b': b'abcd', 'a': 1234}),
    ),
)
def test_contract_constructor_build_transaction_with_constructor_with_argument(
        web3,
        WithConstructorArgumentsContract,
        constructor_args,
        constructor_kwargs,
        address_conversion_func):
    txn_hash = WithConstructorArgumentsContract.constructor(
        *constructor_args, **constructor_kwargs).transact(
        {'from': address_conversion_func(web3.eth.accounts[0])}
    )
    txn = web3.eth.getTransaction(txn_hash)
    nonce = web3.eth.getTransactionCount(web3.eth.coinbase)
    unsent_txn = WithConstructorArgumentsContract.constructor(
        *constructor_args, **constructor_kwargs).buildTransaction({'nonce': nonce})
    assert txn['data'] == unsent_txn['data']

    new_txn_hash = web3.eth.sendTransaction(unsent_txn)
    new_txn = web3.eth.getTransaction(new_txn_hash)
    assert new_txn['data'] == unsent_txn['data']
    assert new_txn['nonce'] == nonce



def test_time_traveling(web3):
    current_block_time = web3.eth.getBlock("pending")['timestamp']

    time_travel_to = current_block_time + 12345

    web3.testing.timeTravel(time_travel_to)

    latest_block_time = web3.eth.getBlock("pending")['timestamp']
    assert latest_block_time >= time_travel_to



# -*- coding: utf-8 -*-

import pytest

from web3.utils.toolz import (
    dissoc,
)

# Ignore warning in pyethereum 1.6 - will go away with the upgrade
pytestmark = pytest.mark.filterwarnings("ignore:implicit cast from 'char *'")


@pytest.fixture()
def math_contract(web3, MathContract, address_conversion_func):
    deploy_txn = MathContract.constructor().transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    math_contract_address = address_conversion_func(deploy_receipt['contractAddress'])
    _math_contract = MathContract(address=math_contract_address)
    assert _math_contract.address == math_contract_address
    return _math_contract


@pytest.fixture()
def fallback_function_contract(web3, FallballFunctionContract, address_conversion_func):
    deploy_txn = FallballFunctionContract.constructor().transact()
    deploy_receipt = web3.eth.waitForTransactionReceipt(deploy_txn)
    assert deploy_receipt is not None
    fallback_contract_address = address_conversion_func(deploy_receipt['contractAddress'])
    _fallback_contract = FallballFunctionContract(address=fallback_contract_address)
    assert _fallback_contract.address == fallback_contract_address
    return _fallback_contract


def test_build_transaction_with_contract_no_arguments(web3, math_contract, buildTransaction):
    txn = buildTransaction(contract=math_contract, contract_function='increment')
    assert dissoc(txn, 'gas') == {
        'to': math_contract.address,
        'data': '0xd09de08a',
        'value': 0,
        'gasPrice': 1,
        'chainId': None,
    }


def test_build_transaction_with_contract_fallback_function(web3, fallback_function_contract):
    txn = fallback_function_contract.fallback.buildTransaction()
    assert dissoc(txn, 'gas') == {
        'to': fallback_function_contract.address,
        'data': '0x',
        'value': 0,
        'gasPrice': 1,
        'chainId': None,
    }


def test_build_transaction_with_contract_class_method(
        web3,
        MathContract,
        math_contract,
        buildTransaction):
    txn = buildTransaction(
        contract=MathContract,
        contract_function='increment',
        tx_params={'to': math_contract.address},
    )
    assert dissoc(txn, 'gas') == {
        'to': math_contract.address,
        'data': '0xd09de08a',
        'value': 0,
        'gasPrice': 1,
        'chainId': None,
    }


def test_build_transaction_with_contract_default_account_is_set(
        web3,
        math_contract,
        buildTransaction):
    txn = buildTransaction(contract=math_contract, contract_function='increment')
    assert dissoc(txn, 'gas') == {
        'to': math_contract.address,
        'data': '0xd09de08a',
        'value': 0,
        'gasPrice': 1,
        'chainId': None,
    }


def test_build_transaction_with_gas_price_strategy_set(web3, math_contract, buildTransaction):
    def my_gas_price_strategy(web3, transaction_params):
        return 5
    web3.eth.setGasPriceStrategy(my_gas_price_strategy)
    txn = buildTransaction(contract=math_contract, contract_function='increment')
    assert dissoc(txn, 'gas') == {
        'to': math_contract.address,
        'data': '0xd09de08a',
        'value': 0,
        'gasPrice': 5,
        'chainId': None,
    }


def test_build_transaction_with_contract_data_supplied_errors(web3,
                                                              math_contract,
                                                              buildTransaction):
    with pytest.raises(ValueError):
        buildTransaction(contract=math_contract,
                         contract_function='increment',
                         tx_params={'data': '0x000'})


def test_build_transaction_with_contract_to_address_supplied_errors(web3,
                                                                    math_contract,
                                                                    buildTransaction):
    with pytest.raises(ValueError):
        buildTransaction(contract=math_contract,
                         contract_function='increment',
                         tx_params={'to': '0xb2930B35844a230f00E51431aCAe96Fe543a0347'})


@pytest.mark.parametrize(
    'transaction_args,method_args,method_kwargs,expected,skip_testrpc',
    (
        (
            {}, (5,), {}, {
                'data': '0x7cf5dab00000000000000000000000000000000000000000000000000000000000000005',  # noqa: E501
                'value': 0, 'gasPrice': 1, 'chainId': None,
            }, False
        ),
        (
            {'gas': 800000}, (5,), {}, {
                'data': '0x7cf5dab00000000000000000000000000000000000000000000000000000000000000005',  # noqa: E501
                'value': 0, 'gasPrice': 1, 'chainId': None,
            }, False
        ),
        (
            {'gasPrice': 21000000000}, (5,), {}, {
                'data': '0x7cf5dab00000000000000000000000000000000000000000000000000000000000000005',  # noqa: E501
                'value': 0, 'gasPrice': 21000000000, 'chainId': None,
            }, False
        ),
        (
            {'nonce': 7}, (5,), {}, {
                'data': '0x7cf5dab00000000000000000000000000000000000000000000000000000000000000005',  # noqa: E501
                'value': 0, 'gasPrice': 1, 'nonce': 7, 'chainId': None,
            }, True
        ),
        (
            {'value': 20000}, (5,), {}, {
                'data': '0x7cf5dab00000000000000000000000000000000000000000000000000000000000000005',  # noqa: E501
                'value': 20000, 'gasPrice': 1, 'chainId': None,
            }, False
        ),
    ),
    ids=[
        'Standard',
        'Explicit Gas',
        'Explicit Gas Price',
        'Explicit Nonce',
        'With Value',
    ]
)
def test_build_transaction_with_contract_with_arguments(web3, skip_if_testrpc, math_contract,
                                                        transaction_args,
                                                        method_args,
                                                        method_kwargs,
                                                        expected,
                                                        skip_testrpc,
                                                        buildTransaction):
    if skip_testrpc:
        skip_if_testrpc(web3)

    txn = buildTransaction(contract=math_contract,
                           contract_function='increment',
                           func_args=method_args,
                           func_kwargs=method_kwargs,
                           tx_params=transaction_args)
    expected['to'] = math_contract.address
    assert txn is not None
    if 'gas' in transaction_args:
        assert txn['gas'] == transaction_args['gas']
    else:
        assert 'gas' in txn
    assert dissoc(txn, 'gas') == expected



import pytest
import socket

from flaky import (
    flaky,
)

from web3.utils.module_testing import (
    EthModuleTest,
    ParityModuleTest as TraceModuleTest,
    PersonalModuleTest,
    Web3ModuleTest,
)

# some tests appear flaky with Parity v1.10.x
MAX_FLAKY_RUNS = 3


def get_open_port():
    sock = socket.socket()
    sock.bind(('127.0.0.1', 0))
    port = sock.getsockname()[1]
    sock.close()
    return str(port)


class ParityWeb3ModuleTest(Web3ModuleTest):
    def _check_web3_clientVersion(self, client_version):
        assert client_version.startswith('Parity/')


class ParityEthModuleTest(EthModuleTest):
    def test_eth_getBlockByNumber_pending(self, web3):
        pytest.xfail('Parity dropped "pending" option in 1.11.1')
        super().test_eth_getBlockByNumber_pending(web3)

    def test_eth_uninstallFilter(self, web3):
        pytest.xfail('eth_uninstallFilter calls to parity always return true')
        super().test_eth_uninstallFilter(web3)

    def test_eth_replaceTransaction(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction(web3, unlocked_account)

    def test_eth_replaceTransaction_already_mined(self, web3, unlocked_account):
        pytest.xfail('Parity is not setup to auto mine')
        super().test_eth_replaceTransaction_already_mined(web3, unlocked_account)

    def test_eth_replaceTransaction_incorrect_nonce(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_incorrect_nonce(web3, unlocked_account)

    def test_eth_replaceTransaction_gas_price_too_low(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_too_low(web3, unlocked_account)

    def test_eth_replaceTransaction_gas_price_defaulting_minimum(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_defaulting_minimum(web3, unlocked_account)

    def test_eth_replaceTransaction_gas_price_defaulting_strategy_higher(self,
                                                                         web3,
                                                                         unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_defaulting_strategy_higher(
            web3, unlocked_account
        )

    def test_eth_replaceTransaction_gas_price_defaulting_strategy_lower(self,
                                                                        web3,
                                                                        unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_replaceTransaction_gas_price_defaulting_strategy_lower(
            web3, unlocked_account
        )

    def test_eth_modifyTransaction(self, web3, unlocked_account):
        pytest.xfail('Needs ability to efficiently control mining')
        super().test_eth_modifyTransaction(web3, unlocked_account)

    @flaky(max_runs=MAX_FLAKY_RUNS)
    def test_eth_getTransactionReceipt_unmined(self, web3, unlocked_account):
        # Parity diverges from json-rpc spec and retrieves pending block
        # transactions with getTransactionReceipt.
        txn_hash = web3.eth.sendTransaction({
            'from': unlocked_account,
            'to': unlocked_account,
            'value': 1,
            'gas': 21000,
            'gasPrice': web3.eth.gasPrice,
        })
        receipt = web3.eth.getTransactionReceipt(txn_hash)
        assert receipt is not None
        assert receipt['blockHash'] is None

    def test_eth_getLogs_with_logs_none_topic_args(self, web3):
        pytest.xfail("Parity matches None to asbent values")
        super().test_eth_getLogs_with_logs_none_topic_args(web3)

    @flaky(max_runs=MAX_FLAKY_RUNS)
    def test_eth_call_old_contract_state(self, web3, math_contract, unlocked_account):
        start_block = web3.eth.getBlock('latest')
        block_num = start_block.number
        block_hash = start_block.hash

        math_contract.functions.increment().transact({'from': unlocked_account})

        # This isn't an incredibly convincing test since we can't mine, and
        # the default resolved block is latest, So if block_identifier was ignored
        # we would get the same result. For now, we mostly depend on core tests.
        # Ideas to improve this test:
        #  - Enable on-demand mining in more clients
        #  - Increment the math contract in all of the fixtures, and check the value in an old block

        block_hash_call_result = math_contract.functions.counter().call(block_identifier=block_hash)
        block_num_call_result = math_contract.functions.counter().call(block_identifier=block_num)
        latest_call_result = math_contract.functions.counter().call(block_identifier='latest')
        default_call_result = math_contract.functions.counter().call()

        assert block_hash_call_result == 0
        assert block_num_call_result == 0
        assert latest_call_result == 0
        assert default_call_result == 0

        # retrieve this right before using - Parity tests might hit a race otherwise
        pending_call_result = math_contract.functions.counter().call(block_identifier='pending')
        # should be '1' on first flaky run, '2' on second, or '3' on third
        if pending_call_result not in range(1, MAX_FLAKY_RUNS + 1):
            raise AssertionError("pending call result was %d!" % pending_call_result)


class ParityPersonalModuleTest(PersonalModuleTest):
    def test_personal_importRawKey(self, web3):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_importRawKey(web3)

    def test_personal_listAccounts(self, web3):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_listAccounts(web3)

    def test_personal_lockAccount(self, web3, unlocked_account):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_lockAccount(web3, unlocked_account)

    def test_personal_unlockAccount_success(self, web3):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_unlockAccount_success(web3)

    def test_personal_unlockAccount_failure(self, web3, unlockable_account):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_unlockAccount_failure(web3, unlockable_account)

    def test_personal_newAccount(self, web3):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_newAccount(web3)

    def test_personal_sendTransaction(
            self,
            web3,
            unlockable_account,
            unlockable_account_pw):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_sendTransaction(
            web3,
            unlockable_account,
            unlockable_account_pw)

    def test_personal_sign_and_ecrecover(
            self,
            web3,
            unlockable_account,
            unlockable_account_pw):
        pytest.xfail('this non-standard json-rpc method is not implemented on parity')
        super().test_personal_sign_and_ecrecover(
            web3,
            unlockable_account,
            unlockable_account_pw)


class ParityTraceModuleTest(TraceModuleTest):
    def test_trace_replay_transaction(self, web3, parity_fixture_data):
        super().test_trace_replay_transaction(web3, parity_fixture_data)

    def test_trace_replay_block_with_transactions(self,
                                                  web3,
                                                  block_with_txn,
                                                  parity_fixture_data):
        pytest.xfail('This method does not exist in older parity versions')
        super().test_trace_replay_block_with_transactions(web3,
                                                          block_with_txn,
                                                          parity_fixture_data)

    def test_trace_replay_block_without_transactions(self, web3, empty_block):
        pytest.xfail('This method does not exist in older parity versions')
        super().test_trace_replay_block_without_transactions(web3, empty_block)

    def test_trace_block(self, web3, block_with_txn):
        super().test_trace_block(web3, block_with_txn)

    def test_trace_transaction(self, web3, parity_fixture_data):
        super().test_trace_transaction(web3, parity_fixture_data)

    def test_trace_call(self, web3, math_contract, math_contract_address):
        super().test_trace_call(web3, math_contract, math_contract_address)

    def test_eth_call_with_0_result(self, web3, math_contract, math_contract_address):
        super().test_eth_call_with_0_result(web3, math_contract, math_contract_address)

    def test_trace_filter(self, web3, txn_filter_params, parity_fixture_data):
        super().test_trace_filter(web3, txn_filter_params, parity_fixture_data)



import itertools
import pytest
import uuid

from web3 import Web3
from web3.middleware import (
    construct_error_generator_middleware,
    construct_result_generator_middleware,
    construct_simple_cache_middleware,
)
from web3.providers.base import (
    BaseProvider,
)
from web3.utils.caching import (
    generate_cache_key,
)


@pytest.fixture
def w3_base():
    return Web3(providers=[BaseProvider()], middlewares=[])


@pytest.fixture
def result_generator_middleware():
    return construct_result_generator_middleware({
        'fake_endpoint': lambda *_: str(uuid.uuid4()),
        'not_whitelisted': lambda *_: str(uuid.uuid4()),
    })


@pytest.fixture
def w3(w3_base, result_generator_middleware):
    w3_base.middleware_stack.add(result_generator_middleware)
    return w3_base


def test_simple_cache_middleware_pulls_from_cache(w3):
    def cache_class():
        return {
            generate_cache_key(('fake_endpoint', [1])): {'result': 'value-a'},
        }

    w3.middleware_stack.add(construct_simple_cache_middleware(
        cache_class=cache_class,
        rpc_whitelist={'fake_endpoint'},
    ))

    assert w3.manager.request_blocking('fake_endpoint', [1]) == 'value-a'


def test_simple_cache_middleware_populates_cache(w3):
    w3.middleware_stack.add(construct_simple_cache_middleware(
        cache_class=dict,
        rpc_whitelist={'fake_endpoint'},
    ))

    result = w3.manager.request_blocking('fake_endpoint', [])

    assert w3.manager.request_blocking('fake_endpoint', []) == result
    assert w3.manager.request_blocking('fake_endpoint', [1]) != result


def test_simple_cache_middleware_does_not_cache_none_responses(w3_base):
    counter = itertools.count()
    w3 = w3_base

    def result_cb(method, params):
        next(counter)
        return None

    w3.middleware_stack.add(construct_result_generator_middleware({
        'fake_endpoint': result_cb,
    }))

    w3.middleware_stack.add(construct_simple_cache_middleware(
        cache_class=dict,
        rpc_whitelist={'fake_endpoint'},
    ))

    w3.manager.request_blocking('fake_endpoint', [])
    w3.manager.request_blocking('fake_endpoint', [])

    assert next(counter) == 2


def test_simple_cache_middleware_does_not_cache_error_responses(w3_base):
    w3 = w3_base
    w3.middleware_stack.add(construct_error_generator_middleware({
        'fake_endpoint': lambda *_: 'msg-{0}'.format(str(uuid.uuid4())),
    }))

    w3.middleware_stack.add(construct_simple_cache_middleware(
        cache_class=dict,
        rpc_whitelist={'fake_endpoint'},
    ))

    with pytest.raises(ValueError) as err_a:
        w3.manager.request_blocking('fake_endpoint', [])
    with pytest.raises(ValueError) as err_b:
        w3.manager.request_blocking('fake_endpoint', [])

    assert str(err_a) != str(err_b)


def test_simple_cache_middleware_does_not_cache_endpoints_not_in_whitelist(w3):
    w3.middleware_stack.add(construct_simple_cache_middleware(
        cache_class=dict,
        rpc_whitelist={'fake_endpoint'},
    ))

    result_a = w3.manager.request_blocking('not_whitelisted', [])
    result_b = w3.manager.request_blocking('not_whitelisted', [])

    assert result_a != result_b



import random

from eth_utils import (
    decode_hex,
)
from flaky import (
    flaky,
)

from web3.utils.threads import (
    Timeout,
)


@flaky(max_runs=3)
def test_miner_setExtra(web3_empty, wait_for_block):
    web3 = web3_empty

    initial_extra = decode_hex(web3.eth.getBlock(web3.eth.blockNumber)['extraData'])

    new_extra_data = b'-this-is-32-bytes-of-extra-data-'

    # sanity
    assert initial_extra != new_extra_data

    web3.miner.setExtra(new_extra_data)

    with Timeout(60) as timeout:
        while True:
            extra_data = decode_hex(web3.eth.getBlock(web3.eth.blockNumber)['extraData'])
            if extra_data == new_extra_data:
                break
            timeout.sleep(random.random())

    after_extra = decode_hex(web3.eth.getBlock(web3.eth.blockNumber)['extraData'])

    assert after_extra == new_extra_data



def test_shh_version(web3, skip_if_testrpc):
    skip_if_testrpc(web3)
    assert web3.shh.version == '6.0'


def test_shh_info(web3, skip_if_testrpc):
    skip_if_testrpc(web3)

    web3.shh.setMaxMessageSize(1024)
    web3.shh.setMinPoW(0.5)

    info = web3.shh.info

    assert len(info) == 4
    assert info["maxMessageSize"] == 1024
    assert info["minPow"] == 0.5



import contextlib
import json
import os
import pprint
import shutil
import signal
import socket
import subprocess
import sys
import tempfile
import time

from eth_utils.curried import (
    apply_formatter_if,
    is_bytes,
    is_checksum_address,
    is_dict,
    is_same_address,
    remove_0x_prefix,
    to_hex,
    to_text,
    to_wei,
)

from web3 import Web3
from web3.utils.module_testing.emitter_contract import (
    EMITTER_ABI,
    EMITTER_BYTECODE,
    EMITTER_ENUM,
)
from web3.utils.module_testing.math_contract import (
    MATH_ABI,
    MATH_BYTECODE,
)
from web3.utils.toolz import (
    merge,
    valmap,
)

COINBASE = '0xdc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd'
COINBASE_PK = '0x58d23b55bc9cdce1f18c2500f40ff4ab7245df9a89505e9b1fa4851f623d241d'

KEYFILE_DATA = '{"address":"dc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd","crypto":{"cipher":"aes-128-ctr","ciphertext":"52e06bc9397ea9fa2f0dae8de2b3e8116e92a2ecca9ad5ff0061d1c449704e98","cipherparams":{"iv":"aa5d0a5370ef65395c1a6607af857124"},"kdf":"scrypt","kdfparams":{"dklen":32,"n":262144,"p":1,"r":8,"salt":"9fdf0764eb3645ffc184e166537f6fe70516bf0e34dc7311dea21f100f0c9263"},"mac":"4e0b51f42b865c15c485f4faefdd1f01a38637e5247f8c75ffe6a8c0eba856f6"},"id":"5a6124e0-10f1-4c1c-ae3e-d903eacb740a","version":3}'  # noqa: E501

KEYFILE_PW = 'web3py-test'
KEYFILE_FILENAME = 'UTC--2017-08-24T19-42-47.517572178Z--dc544d1aa88ff8bbd2f2aec754b1f1e99e1812fd'  # noqa: E501

RAW_TXN_ACCOUNT = '0x39EEed73fb1D3855E90Cbd42f348b3D7b340aAA6'

UNLOCKABLE_PRIVATE_KEY = '0x392f63a79b1ff8774845f3fa69de4a13800a59e7083f5187f1558f0797ad0f01'
UNLOCKABLE_ACCOUNT = '0x12efdc31b1a8fa1a1e756dfd8a1601055c971e13'
UNLOCKABLE_ACCOUNT_PW = KEYFILE_PW


GENESIS_DATA = {
    "nonce": "0xdeadbeefdeadbeef",
    "timestamp": "0x0",
    "parentHash": "0x0000000000000000000000000000000000000000000000000000000000000000",  # noqa: E501
    "extraData": "0x7765623370792d746573742d636861696e",
    "gasLimit": "0x47d5cc",
    "difficulty": "0x01",
    "mixhash": "0x0000000000000000000000000000000000000000000000000000000000000000",  # noqa: E501
    "coinbase": "0x3333333333333333333333333333333333333333",
    "alloc": {
        remove_0x_prefix(COINBASE): {
            'balance': str(to_wei(1000000000, 'ether')),
        },
        remove_0x_prefix(RAW_TXN_ACCOUNT): {
            'balance': str(to_wei(10, 'ether')),
        },
        remove_0x_prefix(UNLOCKABLE_ACCOUNT): {
            'balance': str(to_wei(10, 'ether')),
        },
    },
    "config": {
        "chainId": 131277322940537,  # the string 'web3py' as an integer
        "homesteadBlock": 0,
        "eip155Block": 0,
        "eip158Block": 0
    },
}


def ensure_path_exists(dir_path):
    """
    Make sure that a path exists
    """
    if not os.path.exists(dir_path):
        os.makedirs(dir_path)
        return True
    return False


@contextlib.contextmanager
def tempdir():
    dir_path = tempfile.mkdtemp()
    try:
        yield dir_path
    finally:
        shutil.rmtree(dir_path)


def get_open_port():
    sock = socket.socket()
    sock.bind(('127.0.0.1', 0))
    port = sock.getsockname()[1]
    sock.close()
    return str(port)


def get_geth_binary():
    from geth.install import (
        get_executable_path,
        install_geth,
    )

    if 'GETH_BINARY' in os.environ:
        return os.environ['GETH_BINARY']
    elif 'GETH_VERSION' in os.environ:
        geth_version = os.environ['GETH_VERSION']
        _geth_binary = get_executable_path(geth_version)
        if not os.path.exists(_geth_binary):
            install_geth(geth_version)
        assert os.path.exists(_geth_binary)
        return _geth_binary
    else:
        return 'geth'


def wait_for_popen(proc, timeout):
    start = time.time()
    while time.time() < start + timeout:
        if proc.poll() is None:
            time.sleep(0.01)
        else:
            break


def kill_proc_gracefully(proc):
    if proc.poll() is None:
        proc.send_signal(signal.SIGINT)
        wait_for_popen(proc, 13)

    if proc.poll() is None:
        proc.terminate()
        wait_for_popen(proc, 5)

    if proc.poll() is None:
        proc.kill()
        wait_for_popen(proc, 2)


def wait_for_socket(ipc_path, timeout=30):
    start = time.time()
    while time.time() < start + timeout:
        try:
            sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            sock.connect(ipc_path)
            sock.settimeout(timeout)
        except (FileNotFoundError, socket.error):
            time.sleep(0.01)
        else:
            break


@contextlib.contextmanager
def graceful_kill_on_exit(proc):
    try:
        yield proc
    finally:
        kill_proc_gracefully(proc)


@contextlib.contextmanager
def get_geth_process(geth_binary,
                     datadir,
                     genesis_file_path,
                     geth_ipc_path,
                     geth_port):
    init_datadir_command = (
        geth_binary,
        '--datadir', datadir,
        'init',
        genesis_file_path,
    )
    subprocess.check_output(
        init_datadir_command,
        stdin=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )

    run_geth_command = (
        geth_binary,
        '--datadir', datadir,
        '--ipcpath', geth_ipc_path,
        '--ethash.dagsondisk', '1',
        '--gcmode', 'archive',
        '--nodiscover',
        '--port', geth_port,
        '--etherbase', COINBASE[2:],
    )

    popen_proc = subprocess.Popen(
        run_geth_command,
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        bufsize=1,
    )
    with popen_proc as proc:
        with graceful_kill_on_exit(proc) as graceful_proc:
            yield graceful_proc

        output, errors = proc.communicate()

    print(
        "Geth Process Exited:\n"
        "stdout:{0}\n\n"
        "stderr:{1}\n\n".format(
            to_text(output),
            to_text(errors),
        )
    )


def write_config_json(config, datadir):
    bytes_to_hex = apply_formatter_if(is_bytes, to_hex)
    config_json_dict = valmap(bytes_to_hex, config)

    config_path = os.path.join(datadir, 'config.json')
    with open(config_path, 'w') as config_file:
        config_file.write(json.dumps(config_json_dict))
        config_file.write('\n')


def generate_go_ethereum_fixture(destination_dir):
    with contextlib.ExitStack() as stack:
        datadir = stack.enter_context(tempdir())

        keystore_dir = os.path.join(datadir, 'keystore')
        ensure_path_exists(keystore_dir)
        keyfile_path = os.path.join(keystore_dir, KEYFILE_FILENAME)
        with open(keyfile_path, 'w') as keyfile:
            keyfile.write(KEYFILE_DATA)
        genesis_file_path = os.path.join(datadir, 'genesis.json')
        with open(genesis_file_path, 'w') as genesis_file:
            genesis_file.write(json.dumps(GENESIS_DATA))

        geth_ipc_path_dir = stack.enter_context(tempdir())
        geth_ipc_path = os.path.join(geth_ipc_path_dir, 'geth.ipc')

        geth_port = get_open_port()
        geth_binary = get_geth_binary()

        with get_geth_process(
                geth_binary=geth_binary,
                datadir=datadir,
                genesis_file_path=genesis_file_path,
                geth_ipc_path=geth_ipc_path,
                geth_port=geth_port):

            wait_for_socket(geth_ipc_path)
            web3 = Web3(Web3.IPCProvider(geth_ipc_path))
            chain_data = setup_chain_state(web3)
            # close geth by exiting context
            # must be closed before copying data dir
            verify_chain_state(web3, chain_data)

        # verify that chain state is still valid after closing
        # and re-opening geth
        with get_geth_process(
                geth_binary=geth_binary,
                datadir=datadir,
                genesis_file_path=genesis_file_path,
                geth_ipc_path=geth_ipc_path,
                geth_port=geth_port):

            wait_for_socket(geth_ipc_path)
            web3 = Web3(Web3.IPCProvider(geth_ipc_path))
            verify_chain_state(web3, chain_data)

        static_data = {
            'raw_txn_account': RAW_TXN_ACCOUNT,
            'keyfile_pw': KEYFILE_PW,
        }
        config = merge(chain_data, static_data)
        pprint.pprint(config)
        write_config_json(config, datadir)

        shutil.copytree(datadir, destination_dir)


def verify_chain_state(web3, chain_data):
    receipt = web3.eth.waitForTransactionReceipt(chain_data['mined_txn_hash'])
    latest = web3.eth.getBlock('latest')
    assert receipt.blockNumber <= latest.number


def mine_transaction_hash(web3, txn_hash):
    start_time = time.time()
    web3.miner.start(1)
    while time.time() < start_time + 60:
        receipt = web3.eth.waitForTransactionReceipt(txn_hash)
        if receipt is not None:
            web3.miner.stop()
            return receipt
        else:
            time.sleep(0.1)
    else:
        raise ValueError("Math contract deploy transaction not mined during wait period")


def mine_block(web3):
    origin_block_number = web3.eth.blockNumber

    start_time = time.time()
    web3.miner.start(1)
    while time.time() < start_time + 60:
        block_number = web3.eth.blockNumber
        if block_number > origin_block_number:
            web3.miner.stop()
            return block_number
        else:
            time.sleep(0.1)
    else:
        raise ValueError("No block mined during wait period")


def deploy_contract(web3, name, factory):
    web3.personal.unlockAccount(web3.eth.coinbase, KEYFILE_PW)
    deploy_txn_hash = factory.constructor().transact({'from': web3.eth.coinbase})
    print('{0}_CONTRACT_DEPLOY_HASH: '.format(name.upper()), deploy_txn_hash)
    deploy_receipt = mine_transaction_hash(web3, deploy_txn_hash)
    print('{0}_CONTRACT_DEPLOY_TRANSACTION_MINED'.format(name.upper()))
    contract_address = deploy_receipt['contractAddress']
    assert is_checksum_address(contract_address)
    print('{0}_CONTRACT_ADDRESS:'.format(name.upper()), contract_address)
    return deploy_receipt


def setup_chain_state(web3):
    coinbase = web3.eth.coinbase

    assert is_same_address(coinbase, COINBASE)

    #
    # Math Contract
    #
    math_contract_factory = web3.eth.contract(
        abi=MATH_ABI,
        bytecode=MATH_BYTECODE,
    )
    math_deploy_receipt = deploy_contract(web3, 'math', math_contract_factory)
    assert is_dict(math_deploy_receipt)

    #
    # Emitter Contract
    #
    emitter_contract_factory = web3.eth.contract(
        abi=EMITTER_ABI,
        bytecode=EMITTER_BYTECODE,
    )
    emitter_deploy_receipt = deploy_contract(web3, 'emitter', emitter_contract_factory)
    emitter_contract = emitter_contract_factory(emitter_deploy_receipt['contractAddress'])

    txn_hash_with_log = emitter_contract.functions.logDouble(
        which=EMITTER_ENUM['LogDoubleWithIndex'], arg0=12345, arg1=54321,
    ).transact({
        'from': web3.eth.coinbase,
    })
    print('TXN_HASH_WITH_LOG:', txn_hash_with_log)
    txn_receipt_with_log = mine_transaction_hash(web3, txn_hash_with_log)
    block_with_log = web3.eth.getBlock(txn_receipt_with_log['blockHash'])
    print('BLOCK_HASH_WITH_LOG:', block_with_log['hash'])

    #
    # Empty Block
    #
    empty_block_number = mine_block(web3)
    print('MINED_EMPTY_BLOCK')
    empty_block = web3.eth.getBlock(empty_block_number)
    assert is_dict(empty_block)
    assert not empty_block['transactions']
    print('EMPTY_BLOCK_HASH:', empty_block['hash'])

    #
    # Block with Transaction
    #
    web3.personal.unlockAccount(coinbase, KEYFILE_PW)
    web3.miner.start(1)
    mined_txn_hash = web3.eth.sendTransaction({
        'from': coinbase,
        'to': coinbase,
        'value': 1,
        'gas': 21000,
        'gas_price': web3.eth.gasPrice,
    })
    mined_txn_receipt = mine_transaction_hash(web3, mined_txn_hash)
    print('MINED_TXN_HASH:', mined_txn_hash)
    block_with_txn = web3.eth.getBlock(mined_txn_receipt['blockHash'])
    print('BLOCK_WITH_TXN_HASH:', block_with_txn['hash'])

    geth_fixture = {
        'math_deploy_txn_hash': math_deploy_receipt['transactionHash'],
        'math_address': math_deploy_receipt['contractAddress'],
        'emitter_deploy_txn_hash': emitter_deploy_receipt['transactionHash'],
        'emitter_address': emitter_deploy_receipt['contractAddress'],
        'txn_hash_with_log': txn_hash_with_log,
        'block_hash_with_log': block_with_log['hash'],
        'empty_block_hash': empty_block['hash'],
        'mined_txn_hash': mined_txn_hash,
        'block_with_txn_hash': block_with_txn['hash'],
    }
    return geth_fixture


if __name__ == '__main__':
    fixture_dir = sys.argv[1]
    generate_go_ethereum_fixture(fixture_dir)



import pytest
import subprocess

from eth_utils import (
    to_text,
)


def test_admin_setSolc(web3, skip_if_testrpc):
    skip_if_testrpc(web3)

    try:
        solc_path = subprocess.check_output(['which', 'solc']).strip()
    except subprocess.CalledProcessError:
        pytest.skip('solc binary not found')
    solc_version = subprocess.check_output(['solc', '--version']).strip()

    actual = web3.admin.setSolc(solc_path)
    assert to_text(solc_version) in actual
    assert to_text(solc_path) in actual



from web3.manager import (
    RequestManager,
)
from web3.providers import (
    BaseProvider,
)


def test_provider_property_setter_and_getter():
    provider_a = BaseProvider()
    provider_b = BaseProvider()

    assert provider_a is not provider_b

    manager = RequestManager(None, provider_a)
    assert manager.providers[0] is provider_a

    manager.providers = provider_b

    assert manager.providers[0] is provider_b



import pytest

from web3.exceptions import (
    ValidationError,
)
from web3.providers.websocket import (
    WebsocketProvider,
)


def test_restricted_websocket_kwargs():
    invalid_kwargs = {'uri': 'ws://127.0.0.1:8546'}
    re_exc_message = r'.*found: {0}*'.format(set(invalid_kwargs.keys()))
    with pytest.raises(ValidationError, match=re_exc_message):
        WebsocketProvider(websocket_kwargs=invalid_kwargs)



import pytest

from web3.utils.abi import (
    filter_by_name,
)

ABI_FUNC_1 = {
    "constant": False,
    "inputs": [],
    "name": "func_1",
    "outputs": [],
    "type": "function",
}
ABI_CONSTRUCTOR = {
    "constant": False,
    "inputs": [],
    "type": "constructor",
}
ABI_FALLBACK = {
    "constant": False,
    "type": "fallback",
}
ABI_FUNC_2_SIG_A = {
    "constant": False,
    "inputs": [
        {"name": "a", "type": "uint256"},
    ],
    "name": "func_2",
    "outputs": [],
    "type": "function",
}
ABI_FUNC_2_SIG_B = {
    "constant": False,
    "inputs": [
        {"name": "a", "type": "uint256"},
        {"name": "b", "type": "uint256"},
    ],
    "name": "func_2",
    "outputs": [],
    "type": "function",
}
ABI_FUNC_3 = {
    "constant": False,
    "inputs": [],
    "name": "func_3",
    "outputs": [],
    "type": "function",
}

ABI = [
    ABI_CONSTRUCTOR,
    ABI_FALLBACK,
    ABI_FUNC_1,
    ABI_FUNC_2_SIG_A,
    ABI_FUNC_2_SIG_B,
    ABI_FUNC_3,
]


@pytest.mark.parametrize(
    'name,expected',
    (
        ('func_1', [ABI_FUNC_1]),
        ('func_2', [ABI_FUNC_2_SIG_A, ABI_FUNC_2_SIG_B]),
        ('func_3', [ABI_FUNC_3]),
        ('does_not_exist', []),
    )
)
def test_filter_by_arguments(name, expected):
    actual_matches = filter_by_name(name, ABI)
    assert actual_matches == expected



import pytest

from web3.utils.blocks import (
    is_predefined_block_number,
)


@pytest.mark.parametrize(
    'block_identifier,expected',
    (
        ('earliest', True),
        ('latest', True),
        ('pending', True),
        (1, False),
        ('0x1', False),
    ),
)
def test_is_predefined_block_number(block_identifier, expected):
    actual = is_predefined_block_number(block_identifier)
    assert actual is expected




import pytest

from web3.utils.abi import (
    abi_data_tree,
    map_abi_data,
)
from web3.utils.normalizers import (
    BASE_RETURN_NORMALIZERS,
)


@pytest.mark.parametrize(
    'types, data, expected',
    [
        (
            ["bool[2]", "bytes"],
            [[True, False], b'\x00\xFF'],
            [("bool[2]", [("bool", True), ("bool", False)]), ("bytes", b'\x00\xFF')],
        ),
        (
            ["uint256[]"],
            [[0, 2**256 - 1]],
            [("uint256[]", [("uint256", 0), ("uint256", 2**256 - 1)])],
        ),
    ],
)
def test_abi_data_tree(types, data, expected):
    assert abi_data_tree(types, data) == expected


@pytest.mark.parametrize(
    'types, data, funcs, expected',
    [
        (
            ["bool[2]", "int256"],
            [[True, False], 9876543210],
            [
                lambda typ, dat: (typ, 'Tru-dat') if typ == 'bool' and dat else (typ, dat),
                lambda typ, dat: (typ, hex(dat)) if typ == 'int256' else (typ, dat),
            ],
            [['Tru-dat', False], '0x24cb016ea'],
        ),
        (
            ["address"],
            ['0x5b2063246f2191f18f2675cedb8b28102e957458'],
            BASE_RETURN_NORMALIZERS,
            ['0x5B2063246F2191f18F2675ceDB8b28102e957458'],
        ),
        (
            ["address[]"],
            [['0x5b2063246f2191f18f2675cedb8b28102e957458'] * 2],
            BASE_RETURN_NORMALIZERS,
            [['0x5B2063246F2191f18F2675ceDB8b28102e957458'] * 2],
        ),
    ],
)
def test_map_abi_data(types, data, funcs, expected):
    assert map_abi_data(funcs, types, data) == expected



import pytest

VALID_MANIFEST = {
    'package_name': 'foo',
    'manifest_version': '2',
    'version': '1.0.0',
}


# Returns web3 instance with `pm` module attached
@pytest.fixture
def web3():
    try:
        from web3.pm import PM
    except ModuleNotFoundError as exc:
        assert False, "eth-pm import failed because: %s" % exc
    PM.attach(web3, 'pm')
    return web3


@pytest.mark.xfail(reason="eth-pm will not installed by default until it is stable")
def test_pm_init_with_minimal_manifest(web3):
    pm = web3.pm.get_package_from_manifest(VALID_MANIFEST)
    assert pm.name == 'foo'
